from pathlib import Path
from ..exceptions import InvalidParameterError
from .schedules import read_available_schedules_file
from ..exceptions import InvalidStartDateError
from ..settings import Settings as s
from datetime import datetime
from .working_days import check_working_days_in_future
from warnings import warn
import json

base_path = Path(__file__).parent
sandbox_schedules = (base_path / '../includes/sandbox.csv').resolve()
ecm3_schedules = (base_path / '../includes/ecm3.csv').resolve()


def check_schedule_name(schedule_name):
    """ Read a file containing all available schedules, and return an
    error if the selected schedule_name cannot be found in the file.

    :Args:
     schedule_name - A schedule_name provided by the post.contract()
        function
    """
    schedules = read_available_schedules_file()

    schedule_names = []
    for name in schedules['schedule']:
        schedule_names.append(name['name'].lower())

    if schedule_name.lower() not in schedule_names:
            raise InvalidParameterError(
                '%s is not a valid schedule. The schedules available are as'
                ' follows: %s' % (schedule_name, schedule_names)
            )


def check_termination_type(termination_type):
    """ Check that the termination_type can be found in the approved
    list of termination_types, and if so, return a number to be used by
    other functions.

    :Args:
    termination type - A termination_type provided by the
        post.contract() function
    """
    termination_type_arguments = {
        'take certain number of debits': 0,
        'until further notice': 1,
        'end on exact date': 2,
    }

    if str(termination_type.lower()) not in termination_type_arguments:
        raise InvalidParameterError(
            '%s is not a valid termination type. Please check the termination'
            ' type and re-submit. The available termination types are: \n'
            '- Take certain number of debits\n'
            '- Until further notice\n'
            '- End on exact date' % termination_type
        )
    else:
        return termination_type_arguments[termination_type.lower()]


def check_at_the_end(at_the_end):
    """ Check that the at_the_end argument can be found in the approved
    list of at_the_end arguments, and if so, return a number to be used
    by other functions.

    :Args:
    at_the_end - An at_the_end argument provided by the post.contract()
        function
    """
    at_the_end_arguments = {
        'expire': 0,
        'switch to further notice': 1,
    }

    if str(at_the_end).lower() not in at_the_end_arguments:
        raise InvalidParameterError(
            '%s is not a valid argument for at_the_end. Please check the'
            ' argument re-submit. The available arguments are: \n'
            '- Expire\n'
            '- Switch to further notice\n'
            '- End on exact date' % at_the_end
        )
    else:
        return at_the_end_arguments[at_the_end.lower()]


def check_payment_day_in_week(payment_day_in_week):
    """ Check that the payment_day_in_week argument can be found in the
    set of payment_day_in_week arguments, and throw an error if it
    cannot be found.

    :Args:
    payment_day_in_week - A payment_day_in_week argument provided by the
        post.contract() function
    """
    payment_day_in_week_arguments = [
        'monday',
        'tuesday',
        'wednesday',
        'thursday',
        'friday',
    ]
    if str(payment_day_in_week).lower() not in payment_day_in_week_arguments:
        raise InvalidParameterError(
            '%s is not a valid payment day in week. Please check the payment'
            ' day in week and re-submit. The available arguments are: \n'
            '- monday\n'
            '- tuesday\n'
            '- wednesday\n'
            '- thursday\n'
            '- friday' % payment_day_in_week
        )
    else:
        pass


def check_payment_day_in_month(payment_day_in_month):
    """ Check that the payment_day_in_month argument can be found in the
    set of payment_day_in_month arguments, and throw an error if it
    cannot be found.

    :Args:
    payment_day_in_month - A payment_day_in_month argument provided by
        the post.contract() function
    """
    try:
        if int(payment_day_in_month) not in range(1, 28) \
                and str(payment_day_in_month).lower() != 'last day of month':
            raise InvalidParameterError(
                '%s is not a valid payment day in month. Please check the'
                ' payment day in month and re-submit. The available arguments'
                ' are: \n'
                '- 1\n'
                '- 2\n'
                '- ...\n'
                '- 28\n'
                '- last day of month' % payment_day_in_month
            )
        else:
            pass
    except ValueError:
        raise InvalidParameterError(
            'payment_day_in_month must be an integer.'
        )

def check_payment_month_in_year(payment_month_in_year):
    """ Check that the payment_day_in_year argument can be found in the
    set of payment_day_in_year arguments, and throw an error if it
    cannot be found.

    :Args:
    payment_day_in_year - A payment_day_in_year argument provided by
        the post.contract() function
    """
    if int(payment_month_in_year) not in range(1, 12):
        raise InvalidParameterError(
            '%s is not a valid payment month in year. Please check the payment'
            ' month in year and re-submit. The available arguments are: \n'
            '- 1\n'
            '- 2\n'
            '- 3\n'
            '- ...\n'
            '- 12' % payment_month_in_year
        )
    else:
        pass


def check_number_of_debits(number_of_debits):
    """ Check that the number_of_debits argument is an integer and is
    between 1 and 99. If number_of_debits is not an integer between 1
    and 99, throw and error.

    :Args:
    number_of_debits - A number_of_debits argument provided by
        the post.contract() function
    """
    try:
        number = int(number_of_debits)
        if number < 0 or number > 99:
            raise InvalidParameterError(
                'number_of_debits cannot be lower than 1 or greater than'
                '99.'
            )
        else:
            pass
    except TypeError:
        raise InvalidParameterError(
            'number_of_debits must be a positive integer between 1 and 99.'
        )


def check_start_date(start_date):
    """ Check that the start_date argument is a valid ISO date and is
    at least x working days in the future, where x is the pre-determined
    bacs_processing_days setting. Throw an error if this is not the
    case.

    :Args:
    start_date - A start_date argument provided by the post.contract()
        function
    """
    date_format = '%Y-%m-%d'
    start = datetime.strptime(start_date, date_format).date()
    initial_days_in_future = s.direct_debit_processing_days['initial']
    first_date = check_working_days_in_future(initial_days_in_future)

    if start < first_date:
        if s.contracts['auto_start_date']:
            warn(
                '%s is not a valid start date. The earliest start date '
                'available is %s. This date has automatically been'
                ' selected.'
                % (start_date, first_date)
            )
            return str(first_date)
        raise InvalidStartDateError(
            '%s is not a valid start date. The earliest start date available'
            ' is %s. Please change this and re-submit.'
            % (start_date, first_date)
        )
    else:
        pass


def check_termination_date_is_in_future(termination_date, start_date):
    """ Check that the termination_date argument is a valid ISO date and is
    in the future relative to the start_date. Throw an error if this is not
    the case.

    :Args:
    termination_date - A termination_date argument provided by the
        post.contract() function
    start_date - A start_date argument provided by the post.contract()
        function
    """
    date_format = '%Y-%m-%d'
    start = datetime.strptime(start_date, date_format).date()
    termination = datetime.strptime(termination_date, date_format).date()

    if termination < start:
        raise InvalidParameterError(
            '%s is not a valid termination date. The termination date must be'
            'in the future. The given start date is %s'
            % (termination_date, start_date)
        )
    else:
        return True


def ad_hoc_checker(schedule):
    """ Check the status of a schedule by reading the JSON found in
        the includes folder, and whether or not it is ad-hoc. If it is
        ad-hoc, return True, if not, return False.

    :Args:
    schedule - A schedule_name argument provided by the post.contract()
        function
    """
    schedule_type = None
    if s.current_environment['env'] == 'sandbox':
        schedules_json = sandbox_schedules
    else:
        schedules_json = ecm3_schedules

    with open(schedules_json, 'r') as f:
        data = json.load(f)
        for i in data['schedule']:
            if schedule == i['name']:
                schedule_type = i['ad_hoc']
        if not schedule_type:
            return False
        elif schedule_type:
            return True
        else:
            raise InvalidParameterError(
                'Could not find the schedule.'
            )


def payment_time_frame_checker(schedule):
    """ Check the frequency of a schedule by reading the JSON found in
        the includes folder. Return a number depending on the frequency
        used in other functions for validation.

    :Args:
    schedule - A schedule_name argument provided by the post.contract()
        function
    """
    payment_type = None
    if s.current_environment['env'] == 'sandbox':
        schedules_json = sandbox_schedules
    else:
        schedules_json = ecm3_schedules
    with open(schedules_json, 'r') as f:
        data = json.load(f)
        for i in data['schedule']:
            if schedule == i['name']:
                payment_type = i['frequency']
        if payment_type == 'Weekly':
            return 0
        elif payment_type == 'Monthly':
            return 1
        elif payment_type == 'Annually':
            return 2
        else:
            raise InvalidParameterError(
                'Could not find the schedule.'
            )



"""
eazysdk.excepions
~~~~~~~~~~~~~~~~~

This module contains the set of EazySDK's exceptions
"""
from functools import wraps


class EazySDKException(IOError):
    """ There was an unknown error that occurred while handling your
    request.
    """
    def __init__(self, message, *args):
        self.message = message
        super(EazySDKException, self).__init__(message, *args)


class UnsupportedHTTPMethodError(EazySDKException):
    """ The requested HTTP method is not available to EazySDK.
    """


class SDKNotEnabledError(EazySDKException):
    """ The API key is not enabled.
    """


class ResourceNotFoundError(EazySDKException):
    """ The requested resource could not be found using the parameters.
    """


class InvalidParameterError(EazySDKException):
    """ One or more of the parameters provided are not valid.
    """


class EmptyRequiredParameterError(EazySDKException):
    """ One or more of the required parameters have not been passed into
    the call.
    """


class ParameterNotAllowedError(EazySDKException):
    """ One or more parameters are mutually exclusive
    """


class InvalidEnvironmentError(EazySDKException):
    """ The environment provided was somehow invalid
    """


class InvalidStartDateError(EazySDKException):
    """ The selected start date of the contract is somehow invalid.
    """


class InvalidPaymentDateError(EazySDKException):
    """ The selected payment date is somehow invalid.
    """


class RecordAlreadyExistsError(EazySDKException):
    """ The record trying to be created already exists.
    """


class InvalidSettingsConfiguration(EazySDKException):
    """ A settings is not correct
    """


def common_exceptions_decorator(funct):
    @wraps(funct)
    def wrapper(self, *args, **kwargs):
        func = str(funct(self, *args, **kwargs))
        if 'not supported' in func:
            raise UnsupportedHTTPMethodError(
                'This is a generic error. This error can be caused by'
                'several events, including\n'
                '- The incorrect HTTP method is being used. Ex. You cannot use'
                ' the GET method when attempting to cancel a Direct Debit\n'
                '- The correct HTTP method is being used, but a mandatory '
                'field has been missed'
            )
        elif 'API not enabled' in func or 'does not support' in func:
            raise SDKNotEnabledError(
                'This is a generic error. This can be caused by several '
                'events, including\n'
                '- The API key is not correct.\n'
                '- The API is correct, but the client is not API enabled\n'
                '- The client code is not correct\n'
                '- If performing a POST, this could mean the record you\'re'
                ' trying to post against does not exist.'
            )
        elif 'IIS 8.5 Detailed Error - 404.0 - Not Found' in func\
                or 'No HTTP resource was found' in func:
            raise ResourceNotFoundError(
                'The requested resource could not be found. This is a'
                ' generic error which could be caused by several events,'
                ' including\n'
                '- You are searching against a record that does not exist\n'
                '- You are missing a mandatory parameter in your API call\n'
                '- You are trying to send invalid data to'
                ' EazyCustomerManager.\n'
                '- The provided client code or API key is incorrect.'
            )
        else:
            return func
    return wrapper



from ..exceptions import InvalidParameterError
from re import search as s


def check_postcode_is_valid_uk_format(post_code):
    """ Check a post_code is of valid formatting. This function will not
    verify whether a post_code actually exists, that responsibility lies
    with the client. This function does not take BFPO post_code. If the
    post code is not valid, throw an error.

    :Args:
     post_code - A post_code provided by the post.customer()
        function
    """
    r = s(
        '^([A-Za-z][A-Ha-hJ-Yj-y]?[0-9][A-Za-z0-9]? ?[0-9][A-Za-z]{2}|[Gg][Ii]'
        '[Rr] ?0[Aa]{2})$', post_code
    )
    if not r:
        raise InvalidParameterError(
            '%s is not formatted as a UK post code. Please check the post code'
            ' and re-submit.' % post_code
        )
    else:
        pass


def check_email_address_format(email_address):
    """ Check an email_address is of valid formatting. This function
    will not verify whether a email_address actually exists, instead
    it checks that an email could exist. The responsibility of checking
    that an email actually exists lies with the client. If the email
    formatting is invalid, throw an error.

    :Args:
     email_address - An email_address provided by the post.customer()
        function
    """
    r = s(
        '(^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$)', email_address
    )
    if not r:
        raise InvalidParameterError(
            '%s is not a valid email address. Please check the email address'
            ' and re-submit.' % email_address
        )
    else:
        pass


def check_bank_details_format(account_number, sort_code, account_name):
    """ Check an account_number, sort_code and account_name are all of
    valid formatting. This function does not verify whether or not a
    bank account exists, or whether it could exist. It simply checks the
    length of the three arguments. The responsibility of verifying the
    legitimacy of the account details lies with the client. If any of
    the three values are not the correct formatting, throw an error.

    :Args:
     account_number - An account_number provided by the post.customer()
        function
     sort_code - A sort_code provided by the post.customer()
        function
     account_name - An account_name provided by the post.customer()
        function
    """
    num_r = s(
        '^[0-9]{8}$', account_number
    )
    sort_r = s(
        '^[0-9]{6}$', sort_code
    )
    name_r = s(
        '^[A-Z0-9\-\/& ]{3,18}$', account_name.upper()
    )
    if not num_r:
        raise InvalidParameterError(
            '%s is not formatted as a UK bank account number. UK bank'
            ' account numbers are 8 digits long. Please check the bank'
            ' account number and re-submit.' % account_number
        )
    elif not sort_r:
        raise InvalidParameterError(
            '%s is not formatted as a UK sort code. UK sort codes are 6 digits'
            ' long. Make sure to not include any hyphens. Please check the'
            ' sort code and re-submit.' % sort_code
        )
    elif not name_r:
        raise InvalidParameterError(
            '%s is not formatted as an account holder name. Account holder'
            ' names must be between 3 and 18 characters, contain only capital'
            ' letters (A-7), ampersands (&), hyphens (-), forward'
            ' slashes (/), and spaces ( ).' % account_name
        )
    else:
        pass



from datetime import datetime
from .contract_checks import check_working_days_in_future
from ..settings import Settings as s
from warnings import warn
from ..exceptions import InvalidParameterError


def check_collection_date(collection_date):
    """ Check that the collection_date argument is a valid ISO date and
    is at least x working days in the future, where x is the
    pre-determined bacs_processing_days setting. Throw an error if this
    is not the case.

    :Args:
    collection_date - A collection_date argument provided by the
        post.payment() function
    """
    date_format = '%Y-%m-%d'
    desired_date = datetime.strptime(collection_date, date_format).date()
    ongoing_days = s.direct_debit_processing_days['ongoing']
    first_date = check_working_days_in_future(ongoing_days)

    if desired_date < first_date:
        if s.payments['auto_fix_payment_date']:
            warn(
                '%s is not a valid start date. The earliest start date '
                'available is %s. This date has automatically been'
                ' selected.'
                % (desired_date, first_date)
            )
            return str(first_date)
        raise InvalidParameterError(
            '%s is not a valid collection date. The earliest collection date'
            ' available is %s. Please change this and re-submit.'
            % (desired_date, first_date)
        )
    else:
        pass


def is_credit_allowed_check():
    """ Check whether or not is_credit is enabled for a client. This
    function does not check whether is_credit is enabled through
    EazyCustomerManager, instead it checks the setting in settings.py.
    Regardless of the setting, is_credit will be disallowed if it is
    disallowed on EazyCustomerManager.
    """
    if s.payments['is_credit_allowed']:
        pass
    else:
        warn(
            'This client cannot use the is_credit function. It has been'
            ' automatically disabled.'
        )
        return False


def check_collection_amount(collection_amount):
    """ Check that the collection_amount argument is a float and is
    above 0.00. If collection_amount is not a float and above 0.00,
    throw and error.

    :Args:
    collection_amount - A collection_amount argument provided by
        the post.payment() function
    """
    try:
        if float(collection_amount) >= 0.01:
            pass
        else:
            raise InvalidParameterError(
                'The collection amount must be positive. Zero or'
                ' non-negative amounts are not allowed.'
            )
    except:
        raise InvalidParameterError(
            'collection_amount must be a number.'
        )



from ... import main
from ...exceptions import ParameterNotAllowedError
from ...exceptions import InvalidParameterError
from ...exceptions import InvalidStartDateError
from ...exceptions import ResourceNotFoundError
from ...settings import Settings as s
import unittest


class Test(unittest.TestCase):
    def setUp(self):
        self.eazy = main.EazySDK().post

    def test_ad_hoc_post_contract(self):
        req = self.eazy.contract(
            '310a826b-d095-48e7-a55a-19dba82c566f', 'adhoc_monthly_free',
            '2019-08-15', False, 'Until Further Notice',
            'Switch to further notice'
            )
        self.assertIn('SDKTST-', req)

    def test_ad_hoc_post_contract_auto_fix_start_date(self):
        s.contracts['auto_start_date'] = True
        req = self.eazy.contract(
            '310a826b-d095-48e7-a55a-19dba82c566f', 'adhoc_monthly_free',
            '2018-06-15', False, 'Until Further Notice',
            'Switch to further notice'
        )
        self.assertIn('SDKTST-', req)

    def test_ad_hoc_post_contract_auto_fix_termination_type(self):
        s.contracts['auto_fix_ad_hoc_termination_type'] = True
        req = self.eazy.contract(
                '310a826b-d095-48e7-a55a-19dba82c566f', 'adhoc_monthly_free',
                '2019-08-15', False, 'End on exact date',
                'Switch to further notice',
            )
        self.assertIn('SDKTST-', req)

    def test_ad_hoc_auto_fix_at_the_end(self):
        s.contracts['auto_fix_ad_hoc_at_the_end'] = True
        req = self.eazy.contract(
                '310a826b-d095-48e7-a55a-19dba82c566f', 'adhoc_monthly_free',
                '2019-08-15', False, 'Until further notice',
                'expire',
            )
        self.assertIn('SDKTST-', req)

    def test_ad_hoc_post_contract_gift_aid_true(self):
        req = self.eazy.contract(
            '310a826b-d095-48e7-a55a-19dba82c566f', 'adhoc_monthly_free',
            '2019-08-15', True, 'Until Further Notice',
            'Switch to further notice'
            )
        self.assertIn('SDKTST-', req)

    def test_ad_hoc_post_contract_allow_additional_ref(self):
        req = self.eazy.contract(
            '310a826b-d095-48e7-a55a-19dba82c566f', 'adhoc_monthly_free',
            '2019-08-15', False, 'Until Further Notice',
            'Switch to further notice', additional_reference='test-0001'
            )
        self.assertIn('SDKTST-', req)

    def test_ad_hoc_invalid_customer_throws_error(self):
        with self.assertRaises(ResourceNotFoundError) as e:
            self.eazy.contract(
                '10a826b-d095-48e7-a55a-19dba82c566f', 'adhoc_monthly_free',
                '2019-08-15', False, 'Until Further Notice',
                'Switch to further notice',
                )
        self.assertIn('The requested resource could not', str(e.exception))

    def test_ad_hoc_invalid_start_date_throws_error(self):
        s.contracts['auto_start_date'] = False
        with self.assertRaises(InvalidStartDateError) as e:
            self.eazy.contract(
                '310a826b-d095-48e7-a55a-19dba82c566f', 'adhoc_monthly_free',
                '2018-06-15', False, 'Until Further Notice',
                'Switch to further notice',
            )
        self.assertIn('is not a valid start date', str(e.exception))

    def test_ad_hoc_invalid_termination_type_throws_error(self):
        s.contracts['auto_start_date'] = False
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract(
                '310a826b-d095-48e7-a55a-19dba82c566f', 'adhoc_monthly_free',
                '2019-08-15', False, 'Take certain number of debits',
                'Switch to further notice',
            )
        self.assertIn('termination_type must be set to', str(e.exception))

    def test_ad_hoc_invalid_at_the_end_throws_error(self):
        s.contracts['auto_fix_ad_hoc_at_the_end'] = False
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract(
                '310a826b-d095-48e7-a55a-19dba82c566f', 'adhoc_monthly_free',
                '2019-08-15', False, 'Until further notice',
                'Expire',
            )
        self.assertIn('must be set to Switch to further', str(e.exception))

    def test_ad_hoc_passing_initial_amount_throws_error(self):
        with self.assertRaises(ParameterNotAllowedError) as e:
            self.eazy.contract(
                '310a826b-d095-48e7-a55a-19dba82c566f', 'adhoc_monthly_free',
                '2019-08-15', False, 'Until Further Notice',
                'Switch to further notice', initial_amount='1.00'
            )
        self.assertIn('cannot be passed on ad_hoc contracts', str(e.exception))

    def test_ad_hoc_passing_extra_initial_amount_throws_error(self):
        with self.assertRaises(ParameterNotAllowedError) as e:
            self.eazy.contract(
                '310a826b-d095-48e7-a55a-19dba82c566f', 'adhoc_monthly_free',
                '2019-08-15', False, 'Until Further Notice',
                'Switch to further notice', extra_initial_amount='1.00'
            )
        self.assertIn('cannot be passed on ad_hoc contracts', str(e.exception))

    def test_ad_hoc_passing_final_amount_throws_error(self):
        with self.assertRaises(ParameterNotAllowedError) as e:
            self.eazy.contract(
                '310a826b-d095-48e7-a55a-19dba82c566f', 'adhoc_monthly_free',
                '2019-08-15', False, 'Until Further Notice',
                'Switch to further notice', final_amount='1.00'
            )
        self.assertIn('cannot be passed on ad_hoc contracts', str(e.exception))

    def test_invalid_schedule_name_throws_error(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract(
                '310a826b-d095-48e7-a55a-19dba82c566f', 'not_a_schedule',
                '2019-08-15', False, 'Until Further Notice',
                'Switch to further notice',
            )
        self.assertIn('is not a valid schedule', str(e.exception))

    # All non-adhoc weekly schedules will fail.
    def test_weekly_post_contract_until_further_notice(self):
        req = self.eazy.contract(
            '310a826b-d095-48e7-a55a-19dba82c566f', 'weekly_free',
            '2019-07-15', False, 'Until Further Notice',
            'Switch to further notice', frequency=1,
            payment_amount=10.00, payment_day_in_week='Monday'
            )
        self.assertIn('SDKTST-', req)

    def test_weekly_post_contract_take_certain_number_of_debits(self):
        req = self.eazy.contract(
            '310a826b-d095-48e7-a55a-19dba82c566f', 'weekly_free',
            '2019-07-15', False, 'Take certain number of debits',
            'Switch to further notice', frequency=1,
            payment_amount=10.00, payment_day_in_week='Monday',
            number_of_debits=10
            )
        self.assertIn('SDKTST-', req)

    def test_weekly_post_contract_end_on_exact_date(self):
        req = self.eazy.contract(
            '310a826b-d095-48e7-a55a-19dba82c566f', 'weekly_free',
            '2019-07-15', False, 'End on exact date',
            'Switch to further notice', frequency=1,
            payment_amount=10.00, payment_day_in_week='Monday',
            termination_date='2020-01-15'
            )
        self.assertIn('SDKTST-', req)

    def test_weekly_auto_fix_start_date(self):
        s.contracts['auto_start_date'] = True
        req = self.eazy.contract(
            '310a826b-d095-48e7-a55a-19dba82c566f', 'weekly_free',
            '2019-06-01', False, 'Until Further Notice',
            'Switch to further notice', frequency=1,
            payment_amount=10.00, payment_day_in_Week='Monday'
            )
        self.assertIn('SDKTST-', req)

    def test_weekly_initial_amount(self):
        req = self.eazy.contract(
            '310a826b-d095-48e7-a55a-19dba82c566f', 'weekly_free',
            '2019-07-15', False, 'Until Further Notice',
            'Switch to further notice', frequency=1,
            payment_amount=10.00, payment_day_in_week='Monday',
            initial_amount=10.50
            )
        self.assertIn('SDKTST-', req)

    def test_weekly_extra_initial_amount(self):
        req = self.eazy.contract(
            '310a826b-d095-48e7-a55a-19dba82c566f', 'weekly_free',
            '2019-07-15', False, 'Until Further Notice',
            'Switch to further notice', frequency=1,
            payment_amount=10.00, payment_day_in_week='Monday',
            extra_initial_amount=10.50
            )
        self.assertIn('SDKTST-', req)

    def test_weekly_final_amount(self):
        req = self.eazy.contract(
            '310a826b-d095-48e7-a55a-19dba82c566f', 'weekly_free',
            '2019-07-15', False, 'Until Further Notice',
            'Switch to further notice', frequency=1,
            payment_amount=10.00, payment_day_in_week='Monday',
            final_amount=10.50
            )
        self.assertIn('SDKTST-', req)

    def test_weekly_additional_reference(self):
        req = self.eazy.contract(
            '310a826b-d095-48e7-a55a-19dba82c566f', 'weekly_free',
            '2019-07-15', False, 'Until Further Notice',
            'Switch to further notice', frequency=1,
            payment_amount=10.00, payment_day_in_week='Monday',
            additional_reference='test-0001'
            )
        self.assertIn('SDKTST-', req)

    def test_weekly_invalid_customer_throws_error(self):
        with self.assertRaises(ResourceNotFoundError) as e:
            self.eazy.contract(
                '0a826b-d095-48e7-a55a-19dba82c566f', 'weekly_free',
                '2019-08-15', False, 'End on exact date',
                'Switch to further notice', frequency=1, payment_amount=10.00,
                payment_day_in_week=1, termination_date='2019-10-15'
            )
        self.assertIn('The requested resource could not', str(e.exception))

    def test_weekly_invalid_start_date_throws_error(self):
        s.contracts['auto_start_date'] = False
        with self.assertRaises(InvalidStartDateError) as e:
            self.eazy.contract(
                '310a826b-d095-48e7-a55a-19dba82c566f', 'weekly_free',
                '2019-05-15', False, 'End on exact date',
                'Switch to further notice', frequency=1, payment_amount=10.00,
                payment_day_in_week=1, termination_date='2019-10-15'
            )
        self.assertIn('is not a valid start date', str(e.exception))

    def test_weekly_invalid_payment_date_in_week_throws_error(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract(
                '310a826b-d095-48e7-a55a-19dba82c566f', 'weekly_free',
                '2019-08-15', False, 'End on exact date',
                'Switch to further notice', frequency=1, payment_amount=10.00,
                payment_day_in_week=6, termination_date='2019-10-15'
            )
        self.assertIn('must be set to 15', str(e.exception))

    def test_weekly_frequency_must_be_passed(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract(
                '310a826b-d095-48e7-a55a-19dba82c566f', 'weekly_free',
                '2019-08-15', False, 'End on exact date',
                'Switch to further notice', payment_amount=10.00,
                payment_day_in_week=1, termination_date='2019-10-15'
            )
        self.assertIn('frequency must be passed', str(e.exception))

    def test_weekly_payment_amount_must_be_passed(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract(
                '310a826b-d095-48e7-a55a-19dba82c566f', 'weekly_free',
                '2019-08-15', False, 'End on exact date',
                'Switch to further notice', frequency=1,
                payment_day_in_week=1, termination_date='2019-10-15'
            )
        self.assertIn('payment amount must be passed', str(e.exception))

    def test_weekly_certain_debits_number_of_debits_must_be_passed(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract(
                '310a826b-d095-48e7-a55a-19dba82c566f', 'weekly_free',
                '2019-08-15', False, 'Take certain number of debits',
                'Switch to further notice', frequency=1,
                payment_amount=10.00, payment_day_in_week=1
            )
        self.assertIn('number_of_debits must be passed', str(e.exception))

    def test_weekly_end_on_date_termination_date_must_be_passed(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract(
                '310a826b-d095-48e7-a55a-19dba82c566f', 'weekly_free',
                '2019-08-15', False, 'End on exact date',
                'Switch to further notice', frequency=1,
                payment_amount=10.00, payment_day_in_week=1
            )
        self.assertIn('termination_date must be passed', str(e.exception))

    def test_weekly_end_on_exact_date_termination_date_cannot_be_less(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract(
                '310a826b-d095-48e7-a55a-19dba82c566f', 'weekly_free',
                '2019-08-15', False, 'End on exact date',
                'Switch to further notice', frequency=1,
                payment_amount=10.00, payment_day_in_week=1,
                termination_date='2019-06-14'
            )
        self.assertIn('is not a valid termination date', str(e.exception))

    def test_monthly_post_contract_until_further_notice(self):
        req = self.eazy.contract(
            '310a826b-d095-48e7-a55a-19dba82c566f', 'monthly_free',
            '2019-08-15', False, 'Until Further Notice',
            'Switch to further notice', frequency=1,
            payment_amount=10.00, payment_day_in_month=15
            )
        self.assertIn('SDKTST-', req)

    def test_monthly_post_contract_take_certain_number_of_debits(self):
        req = self.eazy.contract(
            '310a826b-d095-48e7-a55a-19dba82c566f', 'monthly_free',
            '2019-08-15', False, 'Take certain number of debits',
            'Switch to further notice', frequency=1, payment_amount=10.00,
            payment_day_in_month=15, number_of_debits=10
            )
        self.assertIn('SDKTST-', req)

    def test_monthly_post_contract_end_on_exact_date(self):
        req = self.eazy.contract(
            '310a826b-d095-48e7-a55a-19dba82c566f', 'monthly_free',
            '2019-08-15', False, 'End on exact date',
            'Switch to further notice', frequency=1, payment_amount=10.00,
            payment_day_in_month=15, termination_date='2019-10-15'
            )
        self.assertIn('SDKTST-', req)

    def test_monthly_auto_fix_start_date(self):
        s.contracts['auto_start_date'] = True
        req = self.eazy.contract(
            '310a826b-d095-48e7-a55a-19dba82c566f', 'monthly_free',
            '2019-06-01', False, 'Until Further Notice',
            'Switch to further notice', frequency=1,
            payment_amount=10.00, payment_day_in_month=13
            )
        self.assertIn('SDKTST-', req)

    def test_monthly_auto_fix_payment_day_in_month(self):
        s.contracts['auto_fix_payment_day_in_month'] = True
        req = self.eazy.contract(
            '310a826b-d095-48e7-a55a-19dba82c566f', 'monthly_free',
            '2019-06-22', False, 'Until Further Notice',
            'Switch to further notice', frequency=1,
            payment_amount=10.00, payment_day_in_month=1
            )
        self.assertIn('SDKTST-', req)

    def test_monthly_initial_amount(self):
        req = self.eazy.contract(
            '310a826b-d095-48e7-a55a-19dba82c566f', 'monthly_free',
            '2019-08-15', False, 'Until Further Notice',
            'Switch to further notice', frequency=1,
            payment_amount=10.00, payment_day_in_month=15, initial_amount=10.50
            )
        self.assertIn('SDKTST-', req)

    def test_monthly_extra_initial_amount(self):
        req = self.eazy.contract(
            '310a826b-d095-48e7-a55a-19dba82c566f', 'monthly_free',
            '2019-08-15', False, 'Until Further Notice',
            'Switch to further notice', frequency=1,
            payment_amount=10.00, payment_day_in_month=15,
            extra_initial_amount=10.50
            )
        self.assertIn('SDKTST-', req)

    def test_monthly_final_amount(self):
        req = self.eazy.contract(
            '310a826b-d095-48e7-a55a-19dba82c566f', 'monthly_free',
            '2019-08-15', False, 'Until Further Notice',
            'Switch to further notice', frequency=1,
            payment_amount=10.00, payment_day_in_month=15, final_amount=10.50
            )
        self.assertIn('SDKTST-', req)

    def test_monthly_additional_reference(self):
        req = self.eazy.contract(
            '310a826b-d095-48e7-a55a-19dba82c566f', 'monthly_free',
            '2019-08-15', False, 'Until Further Notice',
            'Switch to further notice', frequency=1,
            payment_amount=10.00, payment_day_in_month=15,
            additional_reference='test-004'
            )
        self.assertIn('SDKTST-', req)

    def test_monthly_invalid_customer_throws_error(self):
        with self.assertRaises(ResourceNotFoundError) as e:
            self.eazy.contract(
                '10a826b-d095-48e7-a55a-19dba82c566f', 'monthly_free',
                '2019-08-15', False, 'End on exact date',
                'Switch to further notice', frequency=1, payment_amount=10.00,
                payment_day_in_month=15, termination_date='2019-10-15'
            )
        self.assertIn('The requested resource could not', str(e.exception))

    def test_monthly_invalid_schedule_throws_error(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract(
                '310a826b-d095-48e7-a55a-19dba82c566f', 'not_a_schedule',
                '2019-08-15', False, 'End on exact date',
                'Switch to further notice', frequency=1, payment_amount=10.00,
                payment_day_in_month=15, termination_date='2019-10-15'
            )
        self.assertIn('is not a valid schedule', str(e.exception))

    def test_monthly_invalid_start_date_throws_error(self):
        s.contracts['auto_start_date'] = False
        with self.assertRaises(InvalidStartDateError) as e:
            self.eazy.contract(
                '310a826b-d095-48e7-a55a-19dba82c566f', 'monthly_free',
                '2019-05-15', False, 'End on exact date',
                'Switch to further notice', frequency=1, payment_amount=10.00,
                payment_day_in_month=15, termination_date='2019-10-15'
            )
        self.assertIn('is not a valid start date', str(e.exception))

    def test_monthly_invalid_payment_date_in_month_throws_error(self):
        s.contracts['auto_fix_payment_day_in_month'] = False
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract(
                '310a826b-d095-48e7-a55a-19dba82c566f', 'monthly_free',
                '2019-08-15', False, 'End on exact date',
                'Switch to further notice', frequency=1, payment_amount=10.00,
                payment_day_in_month=1, termination_date='2019-10-15'
            )
        self.assertIn('must be set to 15', str(e.exception))

    def test_monthly_frequency_must_be_passed(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract(
                '310a826b-d095-48e7-a55a-19dba82c566f', 'monthly_free',
                '2019-08-15', False, 'End on exact date',
                'Switch to further notice', payment_amount=10.00,
                payment_day_in_month=15, termination_date='2019-10-15'
            )
        self.assertIn('frequency must be passed', str(e.exception))

    def test_monthly_payment_amount_must_be_passed(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract(
                '310a826b-d095-48e7-a55a-19dba82c566f', 'monthly_free',
                '2019-08-15', False, 'End on exact date',
                'Switch to further notice', frequency=1,
                payment_day_in_month=15, termination_date='2019-10-15'
            )
        self.assertIn('payment amount must be passed', str(e.exception))

    def test_monthly_certain_debits_number_of_debits_must_be_passed(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract(
                '310a826b-d095-48e7-a55a-19dba82c566f', 'monthly_free',
                '2019-08-15', False, 'Take certain number of debits',
                'Switch to further notice', frequency=1,
                payment_amount=10.00, payment_day_in_month=15
            )
        self.assertIn('number_of_debits must be passed', str(e.exception))

    def test_monthly_end_on_date_termination_date_must_be_passed(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract(
                '310a826b-d095-48e7-a55a-19dba82c566f', 'monthly_free',
                '2019-08-15', False, 'End on exact date',
                'Switch to further notice', frequency=1,
                payment_amount=10.00, payment_day_in_month=15
            )
        self.assertIn('termination_date must be passed', str(e.exception))

    def test_monthly_end_on_exact_date_termination_date_cannot_be_less(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract(
                '310a826b-d095-48e7-a55a-19dba82c566f', 'monthly_free',
                '2019-08-15', False, 'End on exact date',
                'Switch to further notice', frequency=1,
                payment_amount=10.00, payment_day_in_month=15,
                termination_date='2019-06-14'
            )
        self.assertIn('is not a valid termination date', str(e.exception))

    def test_annual_post_contract_until_further_notice(self):
        req = self.eazy.contract(
            '310a826b-d095-48e7-a55a-19dba82c566f', 'annual_free',
            '2019-08-15', False, 'Until Further Notice',
            'Switch to further notice', frequency=1,
            payment_amount=10.00, payment_day_in_month=15,
            payment_month_in_year=8
            )
        self.assertIn('SDKTST-', req)

    def test_annual_post_contract_take_certain_number_of_debits(self):
        req = self.eazy.contract(
            '310a826b-d095-48e7-a55a-19dba82c566f', 'annual_free',
            '2019-08-15', False, 'Take certain number of debits',
            'Switch to further notice', frequency=1,
            payment_amount=10.00, payment_day_in_month=15,
            payment_month_in_year=8, number_of_debits=10
            )
        self.assertIn('SDKTST-', req)

    def test_annual_post_contract_end_on_exact_date(self):
        req = self.eazy.contract(
            '310a826b-d095-48e7-a55a-19dba82c566f', 'annual_free',
            '2019-08-15', False, 'End on exact date',
            'Switch to further notice', frequency=1, payment_amount=10.00,
            payment_day_in_month=15, payment_month_in_year=8,
            termination_date='2019-10-15'
            )
        self.assertIn('SDKTST-', req)

    def test_annual_auto_fix_start_date(self):
        s.contracts['auto_start_date'] = True
        req = self.eazy.contract(
            '310a826b-d095-48e7-a55a-19dba82c566f', 'annual_free',
            '2019-08-15', False, 'Until Further Notice',
            'Switch to further notice', frequency=1,
            payment_amount=10.00, payment_day_in_month=13,
            payment_month_in_year=8
            )
        self.assertIn('SDKTST-', req)

    def test_annual_auto_fix_payment_day_in_month(self):
        s.contracts['auto_fix_payment_day_in_month'] = True
        req = self.eazy.contract(
            '310a826b-d095-48e7-a55a-19dba82c566f', 'annual_free',
            '2019-08-15', False, 'Until Further Notice',
            'Switch to further notice', frequency=1,
            payment_amount=10.00, payment_day_in_month=1,
            payment_month_in_year=8
            )
        self.assertIn('SDKTST-', req)

    def test_annual_initial_amount(self):
        req = self.eazy.contract(
            '310a826b-d095-48e7-a55a-19dba82c566f', 'annual_free',
            '2019-08-15', False, 'Until Further Notice',
            'Switch to further notice', frequency=1,
            payment_amount=10.00, payment_day_in_month=15,
            initial_amount=10.50, payment_month_in_year=8
            )
        self.assertIn('SDKTST-', req)

    def test_annual_extra_initial_amount(self):
        req = self.eazy.contract(
            '310a826b-d095-48e7-a55a-19dba82c566f', 'annual_free',
            '2019-08-15', False, 'Until Further Notice',
            'Switch to further notice', frequency=1,
            payment_amount=10.00, payment_day_in_month=15,
            extra_initial_amount=10.50, payment_month_in_year=8
            )
        self.assertIn('SDKTST-', req)

    def test_annual_final_amount(self):
        req = self.eazy.contract(
            '310a826b-d095-48e7-a55a-19dba82c566f', 'annual_free',
            '2019-08-15', False, 'Until Further Notice',
            'Switch to further notice', frequency=1,
            payment_amount=10.00, payment_day_in_month=15, final_amount=10.50,
            payment_month_in_year=8
            )
        self.assertIn('SDKTST-', req)

    def test_annual_additional_reference(self):
        req = self.eazy.contract(
            '310a826b-d095-48e7-a55a-19dba82c566f', 'annual_free',
            '2019-08-15', False, 'Until Further Notice',
            'Switch to further notice', frequency=1,
            payment_amount=10.00, payment_day_in_month=15,
            additional_reference='test-004', payment_month_in_year=8
            )
        self.assertIn('SDKTST-', req)

    def test_annual_invalid_customer_throws_error(self):
        with self.assertRaises(ResourceNotFoundError) as e:
            self.eazy.contract(
                '10a826b-d095-48e7-a55a-19dba82c566f', 'annual_free',
                '2019-08-15', False, 'End on exact date',
                'Switch to further notice', frequency=1, payment_amount=10.00,
                payment_day_in_month=15, termination_date='2019-10-15',
                payment_month_in_year=8
            )
        self.assertIn('The requested resource could not', str(e.exception))

    def test_annual_invalid_start_date_throws_error(self):
        s.contracts['auto_start_date'] = False
        with self.assertRaises(InvalidStartDateError) as e:
            self.eazy.contract(
                '310a826b-d095-48e7-a55a-19dba82c566f', 'annual_free',
                '2019-05-15', False, 'End on exact date',
                'Switch to further notice', frequency=1, payment_amount=10.00,
                payment_day_in_month=15, termination_date='2019-10-15',
                payment_month_in_year=5
            )
        self.assertIn('is not a valid start date', str(e.exception))

    def test_annual_invalid_payment_date_in_month_throws_error(self):
        s.contracts['auto_fix_payment_day_in_month'] = False
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract(
                '310a826b-d095-48e7-a55a-19dba82c566f', 'annual_free',
                '2019-08-15', False, 'End on exact date',
                'Switch to further notice', frequency=1, payment_amount=10.00,
                payment_day_in_month=2, termination_date='2019-10-15',
                payment_month_in_year=8
            )
        self.assertIn('must be set to 15', str(e.exception))

    def test_annual_invalid_payment_month_in_year_throws_error(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract(
                '310a826b-d095-48e7-a55a-19dba82c566f', 'annual_free',
                '2019-08-15', False, 'End on exact date',
                'Switch to further notice', frequency=1, payment_amount=10.00,
                payment_day_in_month=15, termination_date='2019-10-15',
                payment_month_in_year=6
            )
        self.assertIn('must be set to 08', str(e.exception))

    def test_annual_payment_amount_must_be_passed(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract(
                '310a826b-d095-48e7-a55a-19dba82c566f', 'annual_free',
                '2019-08-15', False, 'End on exact date',
                'Switch to further notice', frequency=1,
                payment_day_in_month=15, termination_date='2019-10-15',
                payment_month_in_year=8
            )
        self.assertIn('payment amount must be passed', str(e.exception))

    def test_annual_certain_debits_number_of_debits_must_be_passed(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract(
                '310a826b-d095-48e7-a55a-19dba82c566f', 'annual_free',
                '2019-08-15', False, 'Take certain number of debits',
                'Switch to further notice', frequency=1,
                payment_amount=10.00, payment_day_in_month=15,
                payment_month_in_year=8
            )
        self.assertIn('number_of_debits must be passed', str(e.exception))

    def test_annual_end_on_date_termination_date_must_be_passed(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract(
                '310a826b-d095-48e7-a55a-19dba82c566f', 'annual_free',
                '2019-08-15', False, 'End on exact date',
                'Switch to further notice', frequency=1,
                payment_amount=10.00, payment_day_in_month=15,
                payment_month_in_year=8
            )
        self.assertIn('termination_date must be passed', str(e.exception))

    def test_annual_end_on_exact_date_termination_date_cannot_be_less(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract(
                '310a826b-d095-48e7-a55a-19dba82c566f', 'annual_free',
                '2019-08-15', False, 'End on exact date',
                'Switch to further notice', frequency=1,
                payment_amount=10.00, payment_day_in_month=15,
                termination_date='2019-06-14', payment_month_in_year=8
            )
        self.assertIn('is not a valid termination date', str(e.exception))



from ... import main
from ...settings import Settings as s
from ...exceptions import InvalidParameterError
import unittest


class Test(unittest.TestCase):
    def setUp(self):
        self.eazy = main.EazySDK().post

    def test_post_payment_all_valid_details(self):
        req = self.eazy.payment(
            '1802e1dd-a657-428c-b8d0-ba162fc76203', 10.00, '2019-07-15',
            'test comment', False
        )
        self.assertIn('"Error":null', req)

    def test_auto_fix_payment_date(self):
        s.payments['auto_fix_payment_date'] = True
        req = self.eazy.payment(
            '2b62a358-9a1a-4c71-9450-e419e393dcd1', 10.00, '2018-07-15',
            'test comment', False
        )
        self.assertIn('"Error":null', req)

    def test_post_payment_accepts_no_comment(self):
        req = self.eazy.payment(
            '2b62a358-9a1a-4c71-9450-e419e393dcd1', 10.00, '2019-07-15',
            '', False
        )
        self.assertIn('"Error":null', req)

    def test_post_payment_invalid_contract_throws_error(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.payment(
                '1b62a358-9a1a-4c71-9450-e419e393dcd1', 10.00, '2019-07-15',
                '', False
            )
        self.assertIn(
            'The specified contract GUID does not relate', str(e.exception)
        )

    def test_post_payment_non_positive_amount_throws_error(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.payment(
                '1b62a358-9a1a-4c71-9450-e419e393dcd1', 0, '2019-07-15',
                '', False
            )
        self.assertIn(
            'collection_amount must be a number', str(e.exception)
        )

    def test_post_payment_invalid_date_throws_error(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.payment(
                '1b62a358-9a1a-4c71-9450-e419e393dcd1', 1.0, '2018-07-15',
                '', False
            )
        self.assertIn(
            '2018-07-15 is not a valid start date.', str(e.exception)
        )



from ... import main
from ...exceptions import ResourceNotFoundError
from ...exceptions import InvalidParameterError
import unittest
from random import randint


class Test(unittest.TestCase):
    def setUp(self):
        self.eazy = main.EazySDK().patch

    def test_change_contract_amount(self):
        x = randint(1, 1000)
        req = self.eazy.contract_amount(
            '1802e1dd-a657-428c-b8d0-ba162fc76203', x, 'Change contract amount'
            )
        self.assertIn('collection amount has been updated', req)

    def test_amount_cannot_be_empty(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract_amount(
                '1802e1dd-a657-428c-b8d0-ba162fc76203', '',
                'Change contract amount'
            )
        self.assertIn('amount cannot be empty', str(e.exception))

    def test_amount_must_contain_only_numbers(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract_amount(
                '1802e1dd-a657-428c-b8d0-ba162fc76203', 'twenty pounds',
                'Change contract amount'
            )
        self.assertIn(
            'amount must be a number', str(e.exception)
        )

    def test_amount_comment_cannot_be_blank(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract_amount(
                '1802e1dd-a657-428c-b8d0-ba162fc76203', '15.00',
                ''
            )
        self.assertIn(
            'comment cannot be empty', str(e.exception)
        )

    def test_amount_customer_cannot_be_blank(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract_amount(
                '', '15.00', 'Change contract amount'
            )
        self.assertIn(
            ' cannot be empty', str(e.exception)
        )

    def test_amount_invalid_customer_throws_error(self):
        with self.assertRaises(ResourceNotFoundError) as e:
            self.eazy.contract_amount(
                '1802e1dd-a657-428c-b8d0-ba162fc76212', '15.00',
                'Change contract amount'
            )
        self.assertIn(
            'is not a contract belonging to this client', str(e.exception)
        )

    def test_change_collection_day_month(self):
        x = randint(1, 28)
        req = self.eazy.contract_date_monthly(
            '6dfb8179-2f7f-46cb-bc05-fe7f2d36bf36', x, 'New payment day',
            False,
            )
        self.assertIn(
            'Contract 6dfb8179-2f7f-46cb-bc05-fe7f2d36bf36 day updated to', req
        )

    def test_contract_cannot_be_empty_month(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract_date_monthly(
                '', '1', 'New payment day', False,
            )
        self.assertIn('cannot be empty', str(e.exception))

    def test_invalid_contract_throws_error_month(self):
        with self.assertRaises(ResourceNotFoundError) as e:
            self.eazy.contract_date_monthly(
                '6dfb8179-2f7f-46cb-bc05-fe7f2d36bf23', '1', 'New payment day',
                False,
            )
        self.assertIn(
            'is not a contract belonging to this client.', str(e.exception)
        )

    def test_comment_cannot_be_empty_month(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract_date_monthly(
                '6dfb8179-2f7f-46cb-bc05-fe7f2d36bf36', '1', '', False,
            )
        self.assertIn('cannot be empty', str(e.exception))

    def test_patch_next_payment_cannot_be_blank_month(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract_date_monthly(
                '6dfb8179-2f7f-46cb-bc05-fe7f2d36bf36', '1', '', 'Comment',
            )
        self.assertIn('cannot be empty', str(e.exception))

    def test_next_payment_amount_if_amend_next_payment_month(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract_date_monthly(
                '6dfb8179-2f7f-46cb-bc05-fe7f2d36bf36', '1', True, 'Comment',
            )
        self.assertIn('cannot be empty if amend_next_payment is'
                      ' set to true.', str(e.exception))

    def test_change_collection_day_annual(self):
        x = randint(1, 28)
        y = randint(1, 12)
        req = self.eazy.contract_date_annually(
            '8998eab6-f4fe-43b8-b718-78b4520e5529', x, y, 'New payment day',
            False,
            )
        self.assertIn(
            'Contract 8998eab6-f4fe-43b8-b718-78b4520e5529 day updated to', req
        )

    def test_contract_cannot_be_empty_annual(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract_date_annually(
                '', 1, 1, 'New payment day', False,
            )
        self.assertIn('cannot be empty', str(e.exception))

    def test_invalid_contract_throws_error_annual(self):
        with self.assertRaises(ResourceNotFoundError) as e:
            self.eazy.contract_date_annually(
                '8998eab6-f4fe-43b8-b718-78b4520e5512', 1, 1,
                'New payment day', False,
            )
        self.assertIn(
            'is not a contract belonging to this client.', str(e.exception)
        )

    def test_comment_cannot_be_empty_annual(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract_date_annually(
                '8998eab6-f4fe-43b8-b718-78b4520e5529', 1, 1,
                '', False,
            )
        self.assertIn(
            'cannot be empty.', str(e.exception)
        )

    def test_day_cannot_be_empty_annual(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract_date_annually(
                '8998eab6-f4fe-43b8-b718-78b4520e5529', '', 1,
                'Comment', False,
            )
        self.assertIn(
            'cannot be empty.', str(e.exception)
        )

    def test_month_cannot_be_empty_annual(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract_date_annually(
                '8998eab6-f4fe-43b8-b718-78b4520e5529', 1, '',
                'Comment', False,
            )
        self.assertIn(
            'cannot be empty.', str(e.exception)
        )

    def test_patch_next_payment_cannot_be_blank_annual(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract_date_monthly(
                '8998eab6-f4fe-43b8-b718-78b4520e5529', 1, '', 'Comment', '',
            )
        self.assertIn('cannot be empty', str(e.exception))

    def test_next_payment_amount_if_amend_next_payment_annual(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.contract_date_annually(
                '8998eab6-f4fe-43b8-b718-78b4520e5529', 1, 2,
                'New payment day', True,
            )
        self.assertIn('cannot be empty if amend_next_payment is'
                      ' set to true.', str(e.exception))



import requests
from requests import session
from .settings import Settings as s
from .exceptions import UnsupportedHTTPMethodError
from .exceptions import InvalidEnvironmentError
from json import JSONDecodeError


class Session:
    def __init__(self):
        """
        Creates a new instance of the EazySDK session
        """
        # The environment used for requests sent to EazyCustomerManager
        self.environment = None
        # The client settings for the client, stores API and client code
        self.client_settings = None
        # The client code stored within client_settings
        self.client_code = None
        # The API key stored within client_settings
        self.api_key = None
        # The base URL for requests sent to EazyCustomerManager
        self.base_url = None
        # The headers sent along with the request to EazyCustomerManager
        self.headers = None
        # The parameters sent with the request to EazyCustomerManager
        self.params = None
        # The endpoint (x.com{{/end/point}} of the request URL
        self.endpoint = None
        # The HTTP method of the request sent to EazyCustomerManager
        self.method = None
        # The URL to be sent to EazyCustomerManager
        self.request_url = None
        self.session = session()

    def request(self, method, endpoint, headers=None, params=None):
        """
        Create a request to be sent to EazyCustomerManager

        :Required args:
        - method - The HTTP method of the request to be sent to
            EazyCustomerManager
        - endpoint - The path of the request URL

        :Optional args:
        - headers - Headers sent to EazyCustomerManager along with the
            request
        - params - Parameters to be sent to EazyCustomerManager with the
            request


        :Example:
        request('GET', 'customers')

        :Returns:
        request JSON objects
        """
        # Get the current environment from the settings file
        self.environment = s.current_environment['env'].lower()

        # Ensure the environment is valid
        acceptable_environments = {
            'ecm3',
            'sandbox',
        }
        if self.environment not in acceptable_environments:
            raise InvalidEnvironmentError(
                '%s is not a valid environment. The acceptable environments'
                ' are \n- sandbox - A server for testing the functionality of'
                ' EazyCustomerManager\n - ecm3 - The production'
                ' EazyCustomerManager environment' % self.environment
            )
        # Get the client settings from the settings file
        elif self.environment == 'ecm3':
            self.client_settings = s.ecm3_client_details
        else:
            self.client_settings = s.sandbox_client_details

        # Get the client code from the settings file
        self.client_code = self.client_settings['client_code']
        # Get the API key from the settings file
        self.api_key = self.client_settings['api_key']
        # The base URL for all requests to EazyCustomerManager
        self.base_url = 'https://%s.eazycollect.co.uk/api/v3/client/%s/' \
                        % (self.environment, self.client_code)
        # we need to instantiate the headers object
        self.headers = headers
        # Get the API key from the settings file
        self.api_key = self.client_settings['api_key']
        # Create the headers object
        self.headers = {
            'apiKey': self.api_key,
            'Content-Length': '0',
        }

        # Get the endpoint and append it to the base URL
        self.request_url = self.base_url + endpoint
        # Get the HTTP method of the request
        self.method = method
        # Raise an error if an unsupported HTTP method is used
        if method not in (['GET', 'POST', 'PATCH', 'DELETE']):
            raise UnsupportedHTTPMethodError(
                '%s is not a supported HTTP method when communicating'
                ' with EazyCustomerManager. Valid methods are GET,'
                ' POST, PATCH and DELETE' % method
            )

        # Get the params if there are any
        self.params = self.params
        request_call = getattr(requests, self.method.lower())
        response = request_call(
            self.request_url,
            params=self.params,
            headers=self.headers,
        )
        try:
            if response.text:
                response_json = response.text
            else:
                response_json = {}
        except JSONDecodeError:
            response_json = {}
        return response_json

    def get(self):
        """
        Send a GET request to EazyCustomerManager
        """
        return self.request('GET', self.endpoint, self.params)

    def post(self):
        """
        Send a POST request to EazyCustomerManager
        """
        return self.request('POST', self.endpoint, self.params)

    def patch(self):
        """
        Send a PATCH request to EazyCustomerManager
        """
        return self.request('PATCH', self.endpoint, self.params)

    def delete(self):
        """
        Send a DELETE request to EazyCustomerManager
        """
        return self.request('DELETE', self.endpoint, self.params)



from ...utils import working_days
import unittest
import os
from pathlib import Path

base_path = Path(__file__).parent
holidays_file = (base_path / '../../includes/holidays.csv').resolve()


class Test(unittest.TestCase):
    def test_add_5_working_days(self):
        future = working_days.check_working_days_in_future(5)
        self.assertEqual('2019-06-06', str(future))

    def test_add_10_working_days(self):
        future = working_days.check_working_days_in_future(10)
        self.assertEqual('2019-06-13', str(future))

    def test_add_working_days_overlapping_bank_holiday(self):
        # Next holiday is 2019-08-26.
        future = working_days.check_working_days_in_future(62)
        self.assertEqual('2019-08-27', future)

    def test_add_working_days_starting_on_weekend_starts_from_next_w_day(self):
        future = working_days.check_working_days_in_future(1)
        self.assertEqual('2019-06-03', future)


    def test_read_bank_holidays_file(self):
        x = working_days.read_bank_holiday_file_and_check_if_update_needed()
        self.assertIn('2019-04-19', x)

    def test_create_bank_holidays_file_if_not_exist(self):
        os.remove(holidays_file)
        x = working_days.read_bank_holiday_file_and_check_if_update_needed()
        self.assertIn('2019-04-19', x)


from ... import main
from json import loads as json
from ...exceptions import SDKNotEnabledError
from ...exceptions import ResourceNotFoundError
import unittest


class Test(unittest.TestCase):
    def setUp(self):
        self.eazy = main.EazySDK().get

    def test_searching_customer_with_contracts_returns_contracts(self):
        req = self.eazy.contracts('310a826b-d095-48e7-a55a-19dba82c566f')
        req_json = json(req)
        for record in req_json['Contracts']:
            id = record['Id']
            self.assertIsNotNone(id)
        self.assertGreaterEqual(len(req_json), 1)

    def test_searching_customer_with_no_contracts_returns_no_contracts(self):
        req = self.eazy.contracts('7c1a60c5-af12-4477-a10b-a61770e312a5')
        self.assertIn('does not own any contracts', req)

    def test_searching_invalid_customer_returns_error(self):
        with self.assertRaises(SDKNotEnabledError) as e:
            self.eazy.contracts('7c1a60c5-af12-4477-a10b-a61770e312b6')
        self.assertIn('This is a generic error.', str(e.exception))

    def test_blank_customer_returns_error(self):
        with self.assertRaises(ResourceNotFoundError) as e:
            self.eazy.contracts('')
        self.assertIn('resource could not be found.', str(e.exception))



from ... import main
from ...exceptions import ResourceNotFoundError
import unittest


class Test(unittest.TestCase):
    def setUp(self):
        self.eazy = main.EazySDK().post

    def test_reactivate_contract(self):
        req = self.eazy.reactivate_direct_debit(
            '2b62a358-9a1a-4c71-9450-e419e393dcd1'
        )
        self.assertIn('Contract reactivated', req)

    def test_reactivate_contract_archived_contract_throws_error(self):
        req = self.eazy.archive_contract(
            'a899ced6-a601-4146-92f7-5c8ee40bbf93'
        )
        self.assertIn('is already archived', req)

    def test_reactivate_contract_non_existing_contract_throws_error(self):
        with self.assertRaises(ResourceNotFoundError) as e:
            self.eazy.cancel_direct_debit(
                '99ced6-a601-4146-92f7-5c8ee40bbf93'
            )
        self.assertIn('resource could not be found', str(e.exception))


from ... import main
from ...exceptions import InvalidParameterError
from ...exceptions import SDKNotEnabledError
import unittest


class Test(unittest.TestCase):
    def setUp(self):
        self.eazy = main.EazySDK().patch

    def test_updating_all_customer_details_returns_expected_response(self):
        r = self.eazy.customer(
            '310a826b-d095-48e7-a55a-19dba82c566f', 'test@email.com', 'Mr',
            '1990-01-01', 'Test', 'Test', 'TestCom', '1 Test Road', 'GL52 2NF',
            '14785236', '456123', 'Mr Test Test', '01242694874', '07398745641',
            '0145213458', 'line 2', 'line 3', 'line 4', 'tt'
        )
        self.assertIn('updated successfully', r)

    def test_post_code_too_short_throws_error(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                '310a826b-d095-48e7-a55a-19dba82c566f', post_code='G'
            )

        self.assertIn(
            'is not formatted as a UK post code. Please check the post code'
            ' and re-submit.', str(e.exception)
        )

    def test_post_code_too_long_throws_error(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                '310a826b-d095-48e7-a55a-19dba82c566f', post_code='GL516NBB'
            )

        self.assertIn(
            'is not formatted as a UK post code. Please check the'
            ' post code and re-submit.', str(e.exception)
        )

    def test_post_code_must_contain_letters(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                '310a826b-d095-48e7-a55a-19dba82c566f', post_code='1234567'
            )

        self.assertIn(
            'is not formatted as a UK post code. Please check the'
            ' post code and re-submit.', str(e.exception)
        )

    def test_post_code_must_contain_numbers(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                '310a826b-d095-48e7-a55a-19dba82c566f', post_code='GLTQVMQ'
            )

        self.assertIn(
            'is not formatted as a UK post code. Please check the'
            ' post code and re-submit.', str(e.exception)
        )

    def test_email_must_contain_at_symbol(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                '310a826b-d095-48e7-a55a-19dba82c566f', email='testemail.com'
            )

        self.assertIn(
            'is not a valid email address. Please check the email address'
            ' and re-submit.', str(e.exception)
        )

    def test_email_must_contain_tld(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                '310a826b-d095-48e7-a55a-19dba82c566f', email='test@emailcom'
            )

        self.assertIn(
            'is not a valid email address. Please check the email address'
            ' and re-submit.', str(e.exception)
        )

    def test_email_cannot_contain_special_characters_in_domain(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                '310a826b-d095-48e7-a55a-19dba82c566f',
                email='test@e!m&a$il.com'
            )

        self.assertIn(
            'is not a valid email address. Please check the email address'
            ' and re-submit.', str(e.exception)
        )

    def test_account_number_must_not_contain_letters(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                '310a826b-d095-48e7-a55a-19dba82c566f',
                account_number='1478523a'
            )

        self.assertIn(
            'is not formatted as a UK bank account number. UK bank'
            ' account numbers are 8 digits long. Please check the bank'
            ' account number and re-submit.', str(e.exception)
        )

    def test_account_number_must_not_be_too_short(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                '310a826b-d095-48e7-a55a-19dba82c566f',
                account_number='147852'
            )

        self.assertIn(
            'is not formatted as a UK bank account number. UK bank'
            ' account numbers are 8 digits long. Please check the bank'
            ' account number and re-submit.', str(e.exception)
        )

    def test_account_number_must_not_be_too_long(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                '310a826b-d095-48e7-a55a-19dba82c566f',
                account_number='1478514717'
            )

        self.assertIn(
            'is not formatted as a UK bank account number. UK bank'
            ' account numbers are 8 digits long. Please check the bank'
            ' account number and re-submit.', str(e.exception)
        )

    def test_sort_code_must_not_contain_letters(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                '310a826b-d095-48e7-a55a-19dba82c566f', sort_code='14785a'
            )

        self.assertIn(
            'is not formatted as a UK sort code. UK sort codes are 6'
            ' digits long. Make sure to not include any hyphens. Please check'
            ' the sort code and re-submit.', str(e.exception)
        )

    def test_sort_code_must_not_be_too_short(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                '310a826b-d095-48e7-a55a-19dba82c566f', sort_code='14785')

        self.assertIn(
            'is not formatted as a UK sort code. UK sort codes are 6'
            ' digits long. Make sure to not include any hyphens. Please check'
            ' the sort code and re-submit.', str(e.exception)
        )

    def test_sort_code_must_not_be_too_long(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                '310a826b-d095-48e7-a55a-19dba82c566f', sort_code='1478521'
            )

        self.assertIn(
            'is not formatted as a UK sort code. UK sort codes are 6'
            ' digits long. Make sure to not include any hyphens. Please check'
            ' the sort code and re-submit.', str(e.exception)
        )

    def test_sort_code_must_not_contain_hyphens(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                '310a826b-d095-48e7-a55a-19dba82c566f',
                sort_code='14-78-52')

        self.assertIn(
            'is not formatted as a UK sort code. UK sort codes are 6'
            ' digits long. Make sure to not include any hyphens. Please check'
            ' the sort code and re-submit.', str(e.exception)
        )

    def test_account_holder_name_cannot_be_too_long(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                '310a826b-d095-48e7-a55a-19dba82c566f',
                account_holder_name='Mr Test Test Test Test Test'
            )

        self.assertIn(
            'is not formatted as an account holder name. Account holder names'
            ' must be between 3 and 18 characters, contain only capital'
            ' letters (A-7), ampersands (&), hyphens (-), forward slashes'
            ' (/), and spaces ( ).', str(e.exception)
        )

    def test_account_holder_name_cannot_be_too_short(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                '310a826b-d095-48e7-a55a-19dba82c566f',
                account_holder_name='Mr')

        self.assertIn(
            'is not formatted as an account holder name. Account holder names'
            ' must be between 3 and 18 characters, contain only capital'
            ' letters (A-7), ampersands (&), hyphens (-), forward slashes'
            ' (/), and spaces ( ).', str(e.exception)
        )

    def test_account_holder_name_cannot_contain_special_characters(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                '310a826b-d095-48e7-a55a-19dba82c566f',
                account_holder_name='Mr !^&()$"!')

        self.assertIn(
            'is not formatted as an account holder name. Account holder names'
            ' must be between 3 and 18 characters, contain only capital'
            ' letters (A-7), ampersands (&), hyphens (-), forward slashes'
            ' (/), and spaces ( ).', str(e.exception)
        )

    def test_empty_customer_throws_error(self):
        with self.assertRaises(SDKNotEnabledError):
            self.eazy.customer('', email='test@email.com')

    def test_invalid_customer_guid_throws_error(self):
        with self.assertRaises(SDKNotEnabledError):
            self.eazy.customer(
                '310a826b-f195-48e7-a55a-19dba82c566f', email='test@email.com'
            )




class Settings:
    current_environment = {
        'env': 'sandbox'    # sandbox || ecm3
    }

    sandbox_client_details = {
        'client_code': 'SDKTST',
        'api_key': 'hkZujzFR2907XAtYe6qkKRsBo',
    }

    ecm3_client_details = {
        'client_code': 'LUISCT',
        'api_key': 'xAQdXTPtZPG3QqsUJw73wx1o',
    }



    # Changing these values without prior discussion with EazyCollect may
    # lead to unintended behaviour. If in doubt, contact EazyCollect.
    direct_debit_processing_days = {
        'initial': 10,
        'ongoing': 5,
    }

    contracts = {
        'auto_start_date': False,
        'auto_fix_ad_hoc_termination_type': False,
        'auto_fix_ad_hoc_at_the_end': False,
        'auto_fix_payment_day_in_month': False,
        'auto_fix_payment_month_in_year': False,
    }

    payments = {
        'auto_fix_payment_date': False,
        'is_credit_allowed': False,
    }

    warnings = {
        'customer_search': True,
    }

    other = {
        'bank_holidays_update_days': 30,
        'force_schedule_updates': False,
    }



from .session import Session
from .settings import Settings as s
from warnings import warn
from .exceptions import common_exceptions_decorator
from .exceptions import InvalidParameterError

class Get:
    def __init__(self):
        """
        A collection of GET requests made to the ECM3 API
        """
        self.sdk = Session()

    @common_exceptions_decorator
    def callback_url(self,):
        """
        Get the current callback URL from ECM3

        :Example:
        callback_url()

        :Returns:
        'The current callback url is example.com'
        """
        self.sdk.endpoint = 'BACS/callback'
        response = self.sdk.get()
        # NULL will be returned if a callback URL does not exist
        if str(response) == '{"Message":null}':
            return 'A callback URL has not been set'
        else:
            # Use requests.json to get the part of the response we need.
            return 'The callback URL is {}'.format(response['Message'])

    @common_exceptions_decorator
    def customers(self, email='', title='', search_from='', search_to='',
                  date_of_birth='', customer_reference='', first_name='',
                  surname='', company_name='', post_code='', account_number='',
                  sort_code='', account_holder_name='', home_phone='',
                  work_phone='', mobile_phone='',):
        """
        Search for an individual or a group of customers in ECM3

        NOTE: When searching for customers without using any parameters,
        a warning will be rendered to the user. This can be disabled
        by setting warnings['customer_search'] to false.

        :Args:
        - email - The full email of a customer or group of
             customers in ECM3.
        - title - The title of a single customer or a set of customers
        - search_from - Search for customers added to ECM3 after a given
             date
        - search_to - Search for customers added to ECM3 before a given
             date
        - date_of_birth - The full date of birth of a customer or a set
             of customers
        - customer_reference - The full customer reference of a customer
            or a set of customers
        - first_name - The full or partial first name of a customer or a
            set of customers
        - surname - The full or partial surname of a customer or a set
            of customers
        - company_name - The full or partial company name of a customer
            or a set of customers
        - post code - The full or partial post code of a customer or a
            set of customers
        - account_number - The full account number of a customer
        - sort_code - The full sort code of a customer or a set of
            customers
        - account_holder_name - The full or partial account holder name
            for a customer or a set of customers
        - home phone - The full or partial home phone number of a
            customer or a set of customers
        - work_phone - The full or partial work phone number of a
            customer or a set of customers
            A full or partial work telephone number of a customer
        - mobile_phone - The full or partial mobile phone number of a
            customer or a set of customers

        :Example:
        customers(email='test@email.com', surname='Test')

        :Returns:
        customer json object(s)
        """
        # Get all method arguments
        method_arguments = locals()
        # We will not be passing self into ECM3
        del method_arguments['self']
        # A set of pythonic arguments and their ECM3 counterparts
        conversions = {
            'email': 'email',
            'title': 'title',
            'date_of_birth': 'dateOfBirth',
            'search_from': 'from',
            'search_to': 'to',
            'customer_reference': 'customerRef',
            'first_name': 'firstName',
            'surname': 'surname',
            'company_name': 'companyName',
            'post_code': 'postCode',
            'account_number': 'accountNumber',
            'sort_code': 'bankSortCode',
            'account_holder_name': 'accountHolderName',
            'home_phone': 'homePhoneNumber',
            'work_phone': 'workPhoneNumber',
            'mobile_phone': 'mobilePhoneNumber',
        }

        parameters = {}
        key = None
        if s.warnings['customer_search'] and all(
                value == '' for value in method_arguments.values()):
            warn('Retrieving customers without using any search times '
                 'may take some time.')

        try:
            for key, value in method_arguments.items():
                if value != '':
                    parameters.update({conversions[key]: value})
        except KeyError:
            # Raise custom error if the passed parameter is not defined
            raise InvalidParameterError(
                '%s is not an acceptable argument for this call, refer'
                'to the man page for all available arguments' % key
            )

        self.sdk.endpoint = 'customer'
        self.sdk.params = parameters
        response = self.sdk.get()

        if str(response) != '{"Customers":[]}':
            return response
        else:
            return 'No customers could be found using the search terms:' \
                   '%s' % parameters

    @common_exceptions_decorator
    def contracts(self, customer):
        """
        Return all contracts belonging to a specified customer.

        :Args:
        - customer - The GUID of the customer to be queried.

        :Example:
        contracts('ab09362d-f88e-4ee8-be85-e27e1a6ce06a')

        :Returns:
        contract json objects
        """
        self.sdk.endpoint = 'customer/%s/contract' % customer
        response = self.sdk.get()

        if '"Contracts":[]' in str(response):
            return 'The customer %s does not own any contracts' % customer
        else:
            return response

    @common_exceptions_decorator
    def payments(self, contract, number_of_rows=100):
        """
        Return all payments belonging to a contract.

        :Args:
        - contract - The GUID of the contract to be queried.
        - number_of_rows - The number of payment rows to be returned.
            The maximum number of returned rows is 100. By default, this
            is set to 100.

        :Example:
        payments('ab09362d-f88e-4ee8-be85-e27e1a6ce06a')

        :Returns:
        payment json objects
        """
        self.sdk.endpoint = '/contract/%s/payment' % contract
        self.sdk.params = {'rows': number_of_rows}
        response = self.sdk.get()

        if response == '{"Payments":[]}':
            return 'This contract does not own any payments.'

        return response

    @common_exceptions_decorator
    def payments_single(self, contract, payment):
        """
        Return an individual payment from a specific contract

        :Args:
        - contract - The GUID of the contract to be queried.
        - payment - The GUID of the payment to be queried.

        :Example:f
        payments_single('ab09362d-f88e-4ee8-be85-e27e1a6ce06a',
                        '36bb4f4f-9a7f-4ead-82dc-9295c6fb9e8b')

        :Returns:
        payment json objects
        """
        self.sdk.endpoint = '/contract/%s/payment/%s/' % (contract, payment)
        response = self.sdk.get()

        return response

    @common_exceptions_decorator
    def schedules(self):
        """
        Return all available schedules from ECM3

        NOTE: You should not need to run this command manually without
        exceptional circumstance. The SDK will automatically get a list
        of available schedules when first ran, and place them in the
        /includes folder, named sandbox.csv and ecm3.csv respectively.

        :Example:
        schedules()

        :Returns:
        schedule json objects
        """
        self.sdk.endpoint = 'schedules'
        response = self.sdk.get()
        return response



from .session import Session
from .settings import Settings as s
from warnings import warn
from .utils import customer_checks
from .utils import contract_checks
from .utils import payment_checks
from .exceptions import EazySDKException
from .exceptions import common_exceptions_decorator
from .exceptions import ParameterNotAllowedError
from .exceptions import InvalidParameterError
from .exceptions import ResourceNotFoundError
from .exceptions import RecordAlreadyExistsError


class Post:
    def __init__(self):
        """
        A collection of POST requests made to the ECM3 API
        """
        self.sdk = Session()

    @common_exceptions_decorator
    def callback_url(self, callback_url):
        """
        Create or update the endpoint for data returned from ECM3

        NOTE: We strongly recommend using a HTTPS secured URL as the
        return endpoint.

        :Example:
        callback_url(callback_url='test.com')

        :Returns:
        'The new callback url is example.com'
        """
        self.sdk.endpoint = 'BACS/callback'
        self.sdk.params = {
            'url': callback_url
        }
        response = self.sdk.post()

        if 'ExceptionMessage' in str(response):
            raise EazySDKException(
                'The API serializer is not enabled for this account. Please'
                'contact us on help@eazycollect.co.uk for assistance'
            )
        elif 'Updated' not in str(response):
            raise EazySDKException(
                'For an unexpected reason, the new callback URL was not'
                'created.'
            )
        return 'The new callback URL is %s' % callback_url

    @common_exceptions_decorator
    def customer(self, email, title, customer_reference, first_name, surname,
                 line1, post_code, account_number, sort_code,
                 account_holder_name, line2='', line3='', line4='',
                 company_name='', date_of_birth='', initials='', home_phone='',
                 work_phone='',  mobile_phone='',):
        """
        Create a customer in ECM3

        :Required args:
        - email - The email address of the new customer. This must be
            unique
        - title - The title of the new customer
        - customer_reference - The customer reference of the new
            customer. This must be unique.
        - first_name - The first name of the new customer
        - surname - The surname of the new customer
        - line1 - The first line of the new customers address
        - post_code - The post code of the new customer
        - account_number - The bank account number of the new customer
        - sort_code - The sort code of the new customer
        - account_holder_name - The new customers full name as it
            appears on their bank account

        :Optional args:
        - line2 - The second line of the new customers address
        - line3 - The third line of the new customers address
        - line4 - The fourth line of the new customers address
        - company_name - The name of the company the new customer
            represents
        - date_of_birth - The date of birth of the new customer,
            formatted to ISO standards (YYYY-MM-DD)
        - initials - The initials of the new customer
        - home_phone - The home phone number of the new customer
        - work_phone - The work phone number of the new customer
        - mobile_phone - The mobile phone number of the new customer

        :Example:
        customer(email='test@email.com', title='Mrs', first_name='Jane',
                 surname='Doe', line1='1 Test Lane',
                 post_code='TE5T 2AW', account_number='12345678',
                 sort_code='123456', account_holder_name='Mrs J Doe',)

        :Returns:
        customer json object
        """
        # Get all method arguments
        method_arguments = locals()
        # We will not be passing self into ECM3
        del method_arguments['self']
        # A set of pythonic arguments and their ECM3 counterparts
        conversions = {
            'email': 'email',
            'title': 'title',
            'customer_reference': 'customerRef',
            'first_name': 'firstName',
            'surname': 'surname',
            'line1': 'line1',
            'post_code': 'postCode',
            'account_number': 'accountNumber',
            'sort_code': 'bankSortCode',
            'account_holder_name': 'accountHolderName',
            'line2': 'line2',
            'line3': 'line3',
            'line4': 'line4',
            'company_name': 'companyName',
            'date_of_birth': 'dateOfBirth',
            'initials': 'initials',
            'home_phone': 'homePhone',
            'work_phone': 'workPhone',
            'mobile_phone': 'mobilePhone',
        }
        parameters = {}

        required_parameters = [
            'email',
            'title',
            'customer_reference',
            'first_name',
            'surname',
            'line1',
            'post_code',
            'account_number',
            'sort_code',
            'account_holder_name',
        ]

        key = None
        try:
            for key, value in method_arguments.items():
                if key in required_parameters and value == '':
                    raise InvalidParameterError(
                        '%s cannot be empty.' % key
                    )
                elif value != '':
                    parameters.update({conversions[key]: value})
        except KeyError:
            raise ParameterNotAllowedError(
                '%s is not an acceptable argument for this call, refer'
                'to the man page for all available arguments' % key
            )

        # A collection of tests against required params
        customer_checks.check_postcode_is_valid_uk_format(post_code)
        customer_checks.check_email_address_format(email)
        customer_checks.check_bank_details_format(
            account_number, sort_code, account_holder_name
        )
        self.sdk.endpoint = 'customer'
        self.sdk.params = parameters
        response = self.sdk.post()
        if 'There is an existing Customer with the same Client and Customer' \
           ' ref in the database already' in str(response):
            raise RecordAlreadyExistsError(
                'A customer with the given customer_reference already exists.'
                ' Please change the customer reference and re-submit.'
            )
        return response

    @common_exceptions_decorator
    def contract(self, customer, schedule_name, start_date, gift_aid,
                 termination_type, at_the_end, number_of_debits='',
                 frequency='', initial_amount='', extra_initial_amount='',
                 payment_amount='', final_amount='', payment_month_in_year='',
                 payment_day_in_month='', payment_day_in_week='',
                 termination_date='', additional_reference='',
                 custom_dd_reference='',):
        """
        Create a contract in ECM3

        :Required args:
        - customer - The GUID of the customer the new contract will
            belong to
        - schedule_name - The name of the schedule the new contract will
            belong to. Schedules can be found via api.get.schedules()
        - start_date - The desired start date of the new contract. This
            must be x working days in the future, where x is the
            agreed amount of working days with Eazy Collect.
        - gift_aid - Whether the contract is eligible for gift aid or
            not
        - termination_type - The method of termination for the new
            contract
        - at_the_end - What happens to the new contract after the
            termination event has been triggered

        :Optional args:
        - number_of_debits - Mandatory if termination type is set to
            'Collect certain number of debits'
        - frequency - Mandatory if the new contract is not ad-hoc. This
            parameter allows you to skip periods (e.g a value of 2 would
            collect every 2 weeks or months)
        - initial_amount - Used if the first collection amount is
            different from the rest. Not to be used on ad-hoc contracts.
        - extra_initial_amount - Used for any additional charges, for
            example, a gym registration fee.
        - payment_amount - Mandatory if the contract is not ad-hoc. The
            regular collection amount for the new contract
        - final_amount - Used if the final collection amount is
            different from the rest. Not to be used on ad-hoc contracts.
        - payment_month_in_year - The collection month for annual
            contracts. Jan = 0, Dec = 11
        - payment_day_in_month - The collection day for monthly
            contracts. Accepts 1-28 or 'last day of month'
        - payment_day_in_week - The collection day for weekly contracts.
            0 = Mon, 6 = Sun
        - termination_date - The termination date of the newly created
            contract.
        - additional_reference - An additional reference for the newly
            created contract.
        - custom_dd_reference - A custom Direct Debit reference for the
            newly created contract.

        :Example:
        contract('42217d45-cf22-4430-ab02-acc1f8a2d020',
        'test_schedule', '2019-05-07', False, 'Until further notice',
        'Switch to further notice' additional_reference='test_123')

        :Returns:
        contract json object
        """
        # Perform several validation checks against arguments provided
        method_arguments = locals()
        # We will not need self or customer when passing parameters
        del method_arguments['self']
        del method_arguments['customer']
        # A set of pythonic arguments and their ECM3 counterparts
        conversions = {
            'schedule_name': 'scheduleName',
            'start_date': 'start',
            'gift_aid': 'isGiftAid',
            'termination_type': 'terminationType',
            'at_the_end': 'atTheEnd',
            'number_of_debits': 'numberOfDebits',
            'frequency': 'every',
            'initial_amount': 'initialAmount',
            'extra_initial_amount': 'extraInitialAmount',
            'payment_amount': 'amount',
            'final_amount': 'finalAmount',
            'payment_month_in_year': 'paymentMonthInYear',
            'payment_day_in_month': 'paymentDayInMonth',
            'payment_day_in_week': 'paymentDayInWeek',
            'termination_date': 'terminationDate',
            'additional_reference': 'additionalReference',
            'custom_dd_reference': 'customDirectDebitRef',
        }
        parameters = {}
        # Contract validations
        sched = contract_checks.check_schedule_name(schedule_name)
        ad_hoc = contract_checks.ad_hoc_checker(schedule_name)
        start = contract_checks.check_start_date(start_date)
        term = contract_checks.check_termination_type(termination_type)
        ate = contract_checks.check_at_the_end(at_the_end)

        # Avoid non-assignment warnings
        key = None
        try:
            for key, value in method_arguments.items():
                if value != '':
                    parameters.update({conversions[key]: value})
        except KeyError:
            raise ParameterNotAllowedError(
                '%s is not an acceptable argument for this call, refer'
                ' to the man page for all available arguments' % key
            )

        if start:
            del parameters['start']
            parameters.update({'start': start})

        if not ad_hoc:
            if termination_type.lower() != 'until further notice':
                if s.contracts['auto_fix_ad_hoc_termination_type']:
                    warn('Termination type must be Until Further Notice on'
                         ' ad_hoc contracts. This has been automatically '
                         'applied.')
                    del parameters['terminationType']
                    parameters.update(
                        {'terminationType': 'until further notice'}
                    )
                else:
                    raise InvalidParameterError(
                        'termination_type must be set to Until Further Notice'
                        ' on ad_hoc contracts'
                    )
            if at_the_end.lower() != 'switch to further notice':
                if s.contracts['auto_fix_ad_hoc_at_the_end']:
                    warn('At the end must be Switch To Further Notice on'
                         ' ad_hoc contracts. This has been automatically '
                         'applied.')
                    del parameters['atTheEnd']
                    parameters.update(
                        {'atTheEnd': 'switch to further notice'}
                    )
                else:
                    raise InvalidParameterError(
                        'at_the_end must be set to Switch to further notice on'
                        'ad_hoc contracts'
                    )
            if initial_amount != '':
                raise ParameterNotAllowedError(
                    'initial_amount cannot be passed on ad_hoc contracts.'
                )
            elif extra_initial_amount != '':
                raise ParameterNotAllowedError(
                    'extra_initial_amount cannot be passed on ad_hoc contracts'
                )
            elif final_amount != '':
                raise ParameterNotAllowedError(
                    'final_amount cannot be passed on ad_hoc contracts'
                )
            else:
                pass
        else:
            freq = contract_checks.payment_time_frame_checker(schedule_name)
            if frequency == '':
                raise InvalidParameterError(
                    'frequency must be passed on non-ad_hoc contracts'
                )
            elif payment_amount == '':
                raise InvalidParameterError(
                    'payment amount must be passed on non-ad_hoc contracts'
                )
            elif freq == 0:
                if payment_day_in_week == '':
                    raise InvalidParameterError(
                        'payment_day_in_week must be passed on weekly'
                        ' contracts'
                    )
                else:
                    contract_checks.check_payment_day_in_week(
                        payment_day_in_week
                    )
            elif freq == 1:
                if str(payment_day_in_month) != parameters['start'][8:10]:
                    if s.contracts['auto_fix_payment_day_in_month']:
                        pdim = parameters['start'][8:10]
                        del parameters['paymentDayInMonth']
                        parameters.update({'paymentDayInMonth': pdim})
                    elif payment_day_in_month == '':
                        raise InvalidParameterError(
                            'payment_day_in_month must be passed on monthly'
                            ' contracts'
                        )
                    else:
                        raise InvalidParameterError(
                            'payment_day_in_month must be set to %s if the'
                            ' start date is set to %s'
                            % (parameters['start'][8:10], parameters['start'])
                        )
                else:
                    contract_checks.check_payment_day_in_month(
                        payment_day_in_month
                    )
            elif freq == 2:
                if str(payment_month_in_year) not in parameters['start'][5:7]:
                    if s.contracts['auto_fix_payment_month_in_year']:
                        pmiy = int(parameters['start'][4:6])
                        del parameters['paymentMonthInYear']
                        parameters.update({'paymentMonthInYear': pmiy})
                    elif payment_month_in_year == '':
                        raise InvalidParameterError(
                            'payment_month_in_year must be passed on monthly'
                            ' contracts'
                        )
                    else:
                        raise InvalidParameterError(
                            'payment_month_in_year must be set to %s if the'
                            ' start date is set to %s'
                            % (parameters['start'][5:7], parameters['start'])
                        )
                else:
                    contract_checks.check_payment_month_in_year(
                        payment_month_in_year
                    )

                if str(payment_day_in_month) != parameters['start'][8:10]:
                    if s.contracts['auto_fix_payment_day_in_month']:
                        pdim = parameters['start'][8:10]
                        del parameters['paymentDayInMonth']
                        parameters.update({'paymentDayInMonth': pdim})
                    elif payment_day_in_month == '':
                        raise InvalidParameterError(
                            'payment_day_in_month must be passed on monthly'
                            ' contracts'
                        )
                    else:
                        raise InvalidParameterError(
                            'payment_day_in_month must be set to %s if the'
                            ' start date is set to %s'
                            % (parameters['start'][8:10], parameters['start'])
                        )
            else:
                contract_checks.check_payment_day_in_month(
                    payment_day_in_month
                )

            if term == 0:
                if number_of_debits == '':
                    raise InvalidParameterError(
                        'number_of_debits must be passed if termination_type'
                        ' is take certain number of debits.'
                    )
                else:
                    contract_checks.check_number_of_debits(number_of_debits)
            elif term == 1:
                if ate != 1:
                    raise InvalidParameterError(
                        'at_the_end must be set to Switch to Further Notice if'
                        ' termination_type is set to Until Further Notice'
                    )
            elif term == 2:
                if termination_date == '':
                    raise InvalidParameterError(
                        'termination_date must be passed if termination_type'
                        ' is set to End on Exact Date.'
                    )
                else:
                    date = contract_checks.check_termination_date_is_in_future(
                        termination_date, start_date
                    )
                    if date:
                        pass

        self.sdk.endpoint = 'customer/%s/contract' % customer
        self.sdk.params = parameters
        response = self.sdk.post()
        return response

    @common_exceptions_decorator
    def cancel_direct_debit(self, contract,):
        """
        Cancel a Direct Debit within ECM3

        NOTE: Cancelling a Direct Debit will not cancel the payment
        creation process. The reason being; There are two parts to a
        contract, the schedule and the Direct Debit. Cancelling the
        Direct Debit will cease future payments to the bank, but it will
        generate payments on the system. These payments will return
        unpaid, though any ad-hoc payments must be manually deleted.


        :Required args:
        - contract - The GUID of the Direct Debit to be cancelled

        :Example:
        cancel_direct_debit('42217d45-cf22-4430-ab02-acc1f8a2d020')

        :Returns:
        contract json object
        """
        self.sdk.endpoint = 'contract/%s/cancel' % contract
        response = self.sdk.post()

        if 'Contract not found' in response:
            raise ResourceNotFoundError(
                'Contract %s not found. No action has been taken.'
            )
        else:
            return response

    @common_exceptions_decorator
    def archive_contract(self, contract):
        """
        Archive a Direct Debit within ECM3

        NOTE: Archiving a contract achieves different results to
        cancelling a Direct Debit. First and most importantly, the
        process is irreversible. Once a contract is archived, it can not
        be unarchived. The process flow works like so; The Direct Debit
        is cancelled, any arrears that are outstanding are written off,
        any future scheduled payments are cancelled and finally, the
        contract status is set to archived. Like cancelling a Direct
        Debit, any ad_hoc payments must be manually cancelled.


        :Required args:
        - contract - The GUID of the contract ot be archived

        :Example:
        archive_contract('42217d45-cf22-4430-ab02-acc1f8a2d020')

        :Returns:
        contract json object
        """
        self.sdk.endpoint = 'contract/%s/archive' % contract
        response = self.sdk.post()

        if 'Contract is already archived' in response:
            return(
                'Contract %s is already archived. No action will be taken.' %
                contract
            )
        return response

    @common_exceptions_decorator
    def reactivate_direct_debit(self, contract):
        """
        Reactivate a Direct Debit within ECM3

        NOTE: Reactivating a contract changes the status of a contract from
        ‘Cancelled’ to ‘Pending to activate’. This will sent a new instruction
        to the bank, generating an 0N charge.

        :Required args:
        - contract - The GUID of the contract to be re-activated

        :Example:
        reactivate_direct_debit('42217d45-cf22-4430-ab02-acc1f8a2d020')

        :Returns:
        contract json object
        """
        self.sdk.endpoint = 'contract/%s/reactivate' % contract
        response = self.sdk.post()
        return response

    @common_exceptions_decorator
    def restart_contract(self, contract, termination_type, at_the_end,
                         collection_amount='', initial_amount='',
                         final_amount='', payment_day_in_month='',
                         payment_month_in_year='', additional_reference=''):
        """
        Restart a contract within ECM3

        NOTE: Restarting a contract is fundamentally different to
        reactivating a contract as it can only be performed if the
        following criteria have been met

        - The original contract was a fixed term which has expired
        - The payment schedule has met its end naturally, and the
            contract status has become 'Expired'

        This call adds a new contract onto the end of the previous
        contract, in effect 'recycling' the previous direct debit at
        the bank which can save on Direct Debit setup charges.

        :Required args:
        - contract - The GUID of the contract to be restarted
        - termination_type - The termination type of the restarted
            contract
        - at_the_end - What happens to the new contract after the
            termination event has been triggered

        :Optional args:
        - payment_amount - Mandatory if the contract is not ad-hoc. The
            regular collection amount for the restated contract
        - initial_amount - Used if the first collection amount is
            different from the rest. Not to be used on ad-hoc contracts.
        - final_amount - Used if the final collection amount is
            different from the rest. Not to be used on ad-hoc contracts.
        - payment_day_in_month - The collection day for monthly
            contracts. Accepts 1-28 or 'last day of month'
        - additional_reference - An additional reference for the newly
            created contract.

        :Example:
        restart_contract(
            '42217d45-cf22-4430-ab02-acc1f8a2d020',
            'Until further notice', 'Switch to further notice'
        )

        :Returns:
        contract json object
        """
        # Perform several validation checks against arguments provided
        method_arguments = locals()
        # We will not need self or customer when passing parameters
        del method_arguments['self']
        del method_arguments['contract']
        # A set of pythonic arguments and their ECM3 counterparts
        conversions = {
            'termination_type': 'terminationType',
            'at_the_end': 'atTheEnd',
            'initial_amount': 'initialAmount',
            'collection_amount': 'amount',
            'final_amount': 'finalAmount',
            'payment_day_in_month': 'paymentDayInMonth',
            'payment_month_in_year': 'paymentMonthInYear',
            'termination_date': 'terminationDate',
            'additional_reference': 'additionalReference',
        }
        parameters = {}
        # Contract validations

        # Avoid non-assignment warnings
        key = None
        try:
            for key, value in method_arguments.items():
                if value != '':
                    parameters.update({conversions[key]: value})
        except KeyError:
            raise ParameterNotAllowedError(
                '%s is not an acceptable argument for this call, refer'
                ' to the man page for all available arguments' % key
            )
        self.sdk.params = parameters
        self.sdk.endpoint = 'contract/%s/restart' % contract
        response = self.sdk.post()

        if 'Contract is not expired.' in response:
            return(
                'The contract %s is not expired, no action has been taken'
                % contract
            )

        return response

    @common_exceptions_decorator
    def payment(self, contract, collection_amount, collection_date,
                comment, is_credit=False):
        """
        Create a payment against a contract in ECM3

        :Required args:
        - contract - The GUID of the contact a payment will be made
            against
        - collection_amount - The total amount to be collected from the
            new payment
        - collection_date - The desired start date of the new contract.
            This must be x working days in the future, where x is the
            agreed amount of working days with Eazy Collect.
        - comment - A comment related to the new payment
        - is_credit - If you have your own SUN and you have made prior
            arrangements with Eazy Collect, this may be passed to issue
            a credit to a customer. By default, it is set to false.

        :Example:
        payment('42217d45-cf22-4430-ab02-acc1f8a2d020', '10.00',
                '2019-05-07', 'A new payment')

        :Returns:
        payment json object
        """
        # Perform several validation checks against arguments provided
        method_arguments = locals()
        # We will not need self or contract when passing parameters
        del method_arguments['self']
        del method_arguments['contract']
        # A set of pythonic arguments and their ECM3 counterparts
        conversions = {
            'collection_amount': 'amount',
            'collection_date': 'date',
            'comment': 'comment',
            'is_credit': 'isCredit',
        }
        parameters = {}
        # Contract validations
        payment_checks.check_collection_amount(collection_amount)
        collection = payment_checks.check_collection_date(collection_date)

        if is_credit:
            e = payment_checks.is_credit_allowed_check()
            if not e:
                is_credit = False

        # Avoid non-assignment warnings
        key = None
        try:
            for key, value in method_arguments.items():
                if value != '':
                    parameters.update({conversions[key]: value})
        except KeyError:
            raise ParameterNotAllowedError(
                '%s is not an acceptable argument for this call, refer'
                ' to the man page for all available arguments' % key
            )

        if collection:
            del parameters['date']
            parameters.update({'date': collection})

        self.sdk.endpoint = 'contract/%s/payment' % contract
        self.sdk.params = parameters
        response = self.sdk.post()

        if 'Contract not found' in response:
            raise InvalidParameterError(
                'The specified contract GUID does not relate to a customer'
                ' within ECM3.'
            )

        return response


from ... import main
import unittest
from ...settings import Settings as s
from pathlib import Path
from datetime import datetime
from os import remove

base_path = Path(__file__).parent
sandbox_schedules = (base_path / '../../includes/sandbox.csv').resolve()
ecm3_schedules = (base_path / '../../includes/ecm3.csv').resolve()


class Test(unittest.TestCase):
    def setUp(self):
        self.eazy = main.EazySDK().get

    def test_get_schedules_sandbox_write_to_sandbox_csv(self):
        s.current_environment['env'] = 'sandbox'
        s.other['force_schedule_updates'] = True
        self.eazy.schedules()
        with open(sandbox_schedules, 'r') as f:
            x = f.readline()
        self.assertIn(str(datetime.now().date()), str(x))

    def test_get_schedules_ecm3_write_to_ecm3_csv(self):
        s.current_environment['env'] = 'ecm3'
        s.other['force_schedule_updates'] = False
        self.eazy.schedules()
        with open(ecm3_schedules, 'r') as f:
            x = f.readline()
        self.assertIn(str(datetime.now().date()), str(x))

    def test_get_schedules_sandbox_create_file_if_none_exists(self):
        remove(sandbox_schedules)
        s.current_environment['env'] = 'sandbox'
        s.other['force_schedule_updates'] = True
        self.eazy.schedules()
        with open(sandbox_schedules, 'r') as f:
            x = f.readline()
        self.assertIn(str(datetime.now().date()), str(x))

    def test_get_schedules_ecm3_create_file_if_none_exists(self):
        remove(ecm3_schedules)
        s.current_environment['env'] = 'ecm3'
        s.other['force_schedule_updates'] = True
        self.eazy.schedules()
        with open(sandbox_schedules, 'r') as f:
            x = f.readline()
        self.assertIn(str(datetime.now().date()), str(x))



from .session import Session
from .exceptions import common_exceptions_decorator
from .exceptions import ResourceNotFoundError


class Delete:
    def __init__(self):
        """
        A collection of DELETE requests made to the EazyCustomerManager
        API
        """
        self.sdk = Session()

    @common_exceptions_decorator
    def callback_url(self,):
        """
        Delete the current callback URL from EazyCustomerManager

        :Example:
        callback_url()

        :Returns:
        'Callback URL deleted.'
        """
        self.sdk.endpoint = 'BACS/callback'
        response = self.sdk.delete()
        # NULL will be returned if a callback URL does not exist
        if str(response) == '{"Message":null}':
            return 'An unknown error has occurred.'
        else:
            # Use requests.json to get the part of the response we need.
            return 'Callback URL deleted.'

    @common_exceptions_decorator
    def payment(self, contract, payment, comment):
        """
        Delete a payment from EazyCustomerManager, as long as it hasn't
        already been submitted to BACS.

        :Args:
        - contract - The unique GUID of the contract.
        - payment - The unique GUID of the payment.
        - comment - A comment that can be returned when querying the
                    payment

        :Example:
        customers('ab09362d-f88e-4ee8-be85-e27e1a6ce06a',
                  ')

        :Returns:
        customer json object(s)
        """
        parameters = {
            'comment': comment
        }
        self.sdk.params = parameters
        self.sdk.endpoint = 'contract/%s/payment/%s' % (contract, payment)
        response = self.sdk.delete()

        if 'Payment not found' in response:
            raise ResourceNotFoundError(
                'The payment %s either doesn\'t exist or has already been'
                ' deleted.' % payment
            )
        return response



from ... import main
from ...exceptions import ResourceNotFoundError
import unittest
from json import loads


class Test(unittest.TestCase):
    def setUp(self):
        self.eazy = main.EazySDK().get

    def test_payments_get_all_payments_for_a_contract(self):
        req = self.eazy.payments(
            '2b62a358-9a1a-4c71-9450-e419e393dcd1',
        )
        self.assertIn('be047ad1-e314-4980-9cf6-2bb1d324a41d', req)

    def test_payments_get_specific_number_of_payments_from_a_contract(self):
        req = self.eazy.payments(
            '2b62a358-9a1a-4c71-9450-e419e393dcd1', 2
        )
        r = loads(req)
        self.assertEqual(2, len(r['Payments']))
        self.assertIn('8e194a8e-1d0b-4ac1-bf28-404b1516760f', req)

    def test_payments_get_single_payment_from_guid(self):
        req = self.eazy.payments_single(
            '2b62a358-9a1a-4c71-9450-e419e393dcd1',
            'be047ad1-e314-4980-9cf6-2bb1d324a41d',
        )
        self.assertIn('"Amount":10.00,', req)

    def test_payments_get_all_payments_invalid_contract_throws_error(self):
        with self.assertRaises(ResourceNotFoundError) as e:
            self.eazy.payments(
                'b62a358-9a1a-4c71-9450-e419e393dcd1',
            )
        self.assertIn('resource could not be found.', str(e.exception))

    def test_payments_single_payment_invalid_contract_throws_error(self):
        with self.assertRaises(ResourceNotFoundError) as e:
            self.eazy.payments_single(
                'b62a358-9a1a-4c71-9450-e419e393dcd1',
                'be047ad1-e314-4980-9cf6-2bb1d324a41d',
            )
        self.assertIn('resource could not be found.', str(e.exception))

    def test_payments_single_payment_invalid_payment_throws_error(self):
        with self.assertRaises(ResourceNotFoundError) as e:
            self.eazy.payments_single(
                '2b62a358-9a1a-4c71-9450-e419e393dcd1',
                'e047ad1-e314-4980-9cf6-2bb1d324a41d',
            )
        self.assertIn('resource could not be found.', str(e.exception))

    def test_get_all_payments_no_payments_returns_message(self):
        req = self.eazy.payments(
            'a4d77bc5-8480-4148-8838-1afd7f5bab6b',
        )
        self.assertIn('This contract does not own any payments.', req)



from __future__ import unicode_literals
from __future__ import absolute_import
from .get import Get
from .post import Post
from .patch import Patch
from .delete import Delete
from .settings import Settings

class EazySDK:
    """
    Creates a new instance of the EazySDK
    """
    def __init__(self):
        self.settings = Settings()
        self.get = Get()
        self.post = Post()
        self.patch = Patch()
        self.delete = Delete()



from ... import main
from ...exceptions import ResourceNotFoundError
import unittest


class Test(unittest.TestCase):
    def setUp(self):
        self.eazy = main.EazySDK().delete

    def test_delete_payment(self):
        req = self.eazy.payment(
            '1802e1dd-a657-428c-b8d0-ba162fc76203',
            '26a77d64-f21d-43cc-9dc4-765d3a3fc6c1', 'Deleted',
        )
        self.assertIn('Payment deleted', req)

    def test_delete_payment_non_payment_throws_error(self):
        with self.assertRaises(ResourceNotFoundError) as e:
            self.eazy.payment(
                '1802e1dd-a657-428c-b8d0-ba162fc76203',
                '26a77d64-f21d-43cc-9dc4-765d3a3fc6c1', 'Deleted',
            )
        self.assertIn('either doesn\'t exist', str(e.exception))

    def test_delete_payment_non_contract_throws_error(self):
        with self.assertRaises(ResourceNotFoundError) as e:
            self.eazy.payment(
                '802e1dd-a657-428c-b8d0-ba162fc76203',
                '26a77d64-f21d-43cc-9dc4-765d3a3fc6c1', 'Deleted',
            )
        self.assertIn('resource could not be found', str(e.exception))



from ... import main
import unittest


class Test(unittest.TestCase):
    def setUp(self):
        self.eazy = main.EazySDK().patch

    def test_payment_patch_valid_patch(self):
        req = self.eazy.payment(
            '2b62a358-9a1a-4c71-9450-e419e393dcd1',
            'a75f9829-2753-4f67-aafb-bb24aba27dd1', 10.00, '2019-07-15',
            'test comment'
        )
        self.assertIn('"Payment updated', req)




from pathlib import Path
from datetime import datetime
from ..settings import Settings as s
import json
from ..get import Get


base_path = Path(__file__).parent
sandbox_schedules_file = (base_path / '../includes/sandbox.csv').resolve()
ecm3_schedules_file = (base_path / '../includes/ecm3.csv').resolve()


def read_available_schedules_file():
    """ Read the schedules file currently being used depending on the
    current_environment setting. If the schedules file has not been
    updated in over a year, or the setting to force the schedule updates
    is enabled, update the schedules file by calling
    update_schedules_file(). Return a list of schedules to be used by
    other functions.
    """
    # Check the current environment, and select the appropriate file
    if s.current_environment['env'] == 'sandbox':
        file = sandbox_schedules_file
    else:
        file = ecm3_schedules_file
    # Instantiate the dictionary we will parse JSON into
    schedules_json = {}
    date_today = datetime.now().date()
    day_difference = 0

    with open(file, 'a+') as f:
        f.seek(0)
        try:
            data = json.load(f)
            # ISO date formatting
            date_format = '%Y-%m-%d'
            last_update_date_text = data['last_update_date']['last_updated']
            last_update_date = datetime.strptime(
                last_update_date_text,
                date_format,
            )
            day_difference += date_today.day - last_update_date.day

            # Instantiate the JSON object to be written to
            schedules_json['schedule'] = []
            # Get the schedules from EazyCustomerManager
            schedules = Get().schedules()
            services_list = json.loads(schedules)
            # Read the schedules
            schedule_list = services_list['Services']

            for i in range(len(schedule_list)):
                schedules = (schedule_list[i]['Schedules'])
                for schedule in schedules:
                    # Ad-hoc will appear nowhere else other than
                    # potentially name. It will be in Description
                    # every time, however.
                    if 'AD-HOC Payments' in schedule['Description']:
                        schedule_type = False
                    else:
                        schedule_type = True

                    schedules_json['schedule'].append({
                        'name': schedule['Name'],
                        'ad_hoc': schedule_type,
                        'frequency': schedule['Frequency'],
                    })
                    # We save the date in ISO format, but JSON cannot
                    # parse a date
                schedules_json['last_update_date'] = ({
                    'last_updated': str(datetime.now().date())
                })
            if day_difference >= 365 or s.other['force_schedule_updates']:
                update_schedules_file(schedules_json)

            return schedules_json

        except:
            # Instantiate the JSON object to be written to
            schedules_json['schedule'] = []
            # Get the schedules from EazyCustomerManager
            schedules = Get().schedules()
            services_list = json.loads(schedules)
            # Read the schedules
            schedule_list = services_list['Services']

            for i in range(len(schedule_list)):
                schedules = (schedule_list[i]['Schedules'])
                for schedule in schedules:
                    # Ad-hoc will appear nowhere else other than
                    # potentially name. It will be in Description
                    # every time, however.
                    if 'AD-HOC Payments' in schedule['Description']:
                        schedule_type = False
                    else:
                        schedule_type = True

                    schedules_json['schedule'].append({
                        'name': schedule['Name'],
                        'ad_hoc': schedule_type,
                        'frequency': schedule['Frequency'],
                    })
                    # We save the date in ISO format, but JSON cannot
                    # parse a date
                schedules_json['last_update_date'] = ({
                    'last_updated': str(datetime.now().date())
                })
            update_schedules_file(schedules_json)
            return schedules_json

def update_schedules_file(schedules_json):
    """ Update the schedules file with the list of schedules passed by
    read_available_schedules_file()

    :Args:
    schedules_json - A JSON object of all of the schedules provided by
        read_available_schedules_file()
    """
    if s.current_environment['env'] == 'sandbox':
        file = sandbox_schedules_file
    else:
        file = ecm3_schedules_file

    with open(file, 'w') as f:
        json.dump(schedules_json, f)
    return 'Updated bank holidays file.'



from setuptools import setup
from setuptools import find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="eazysdk",
    version="1.0.0",
    author="Billy Glasbey",
    author_email="help@eazycollect.co.uk",
    description="A Python SDK client to interact with EazyCustomerManager",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/EazyCollectServices/EazyCollectSDK-Python",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ], install_requires=['requests']
)


from ... import main
from json import loads as json
from json import JSONDecodeError
from datetime import datetime
from ...settings import Settings as s
import unittest


class Test(unittest.TestCase):
    def setUp(self):
        self.eazy = main.EazySDK().get

    def test_searching_more_than_one_parameter_narrows_search(self):
        req = self.eazy.customers(
            email='test@email.com', customer_reference='test-000001',
            surname='Test', account_holder_name='Mr Test Test'
        )
        req_json = json(req)
        for record in req_json['Customers']:
            email = record['Email']
            ref = record['CustomerRef']
            surname = record['Surname']
            account_holder_name = record['BankDetail']['AccountHolderName']

            self.assertEqual(email, 'test@email.com')
            self.assertEqual(ref, 'test-000001')
            self.assertEqual(surname, 'Test')
            self.assertEqual(account_holder_name, 'Mr Test Test')

        self.assertEqual(len(req_json), 1)

    def test_searching_more_than_one_parameter_incorrect(self):
        req = self.eazy.customers(
            email='test@email.com', customer_reference='not_a_reference',
            surname='Test', account_holder_name='Mr Test Test'
        )
        try:
            json(req)
            self.fail('You should not see this.')
        except JSONDecodeError:
            self.assertEqual(1, 1)

    def test_search_email_full_email(self):
        req = self.eazy.customers(email='test@email.com')
        req_json = json(req)
        for record in req_json['Customers']:
            email = record['Email']
            self.assertIn('test@email.com', email)
        self.assertEqual(len(req_json), 1)

    def test_search_full_title(self):
        # This will fail. This is a limitation of the API.
        req = self.eazy.customers(title='Mx')
        req_json = json(req)
        for record in req_json['Customers']:
            title = record['Title']
            self.assertIn('Mx', title)
        self.assertGreaterEqual(len(req_json), 1)

    def test_search_partial_title(self):
        # This will fail. This is a limitation of the API.
        req = self.eazy.customers(title='Maste')
        req_json = json(req)
        for record in req_json['Customers']:
            title = record['Title']
            self.assertIn('Maste', title)
        self.assertEqual(len(req_json), 1)

    def test_search_search_from_full_date(self):
        date_format = '%Y-%m-%d'
        req_date = datetime.strptime('2019-01-01', date_format).date()
        req = self.eazy.customers(search_from=req_date)
        self.assertNotIn('No customers could be found', req)

        req_json = json(req)
        for record in req_json['Customers']:
            date_record = record['DateAdded'][0:10]
            date = datetime.strptime(date_record, date_format).date()
            self.assertGreaterEqual(date, req_date)

    def test_search_search_from_month(self):
        date_format = '%Y-%m'
        req_date = datetime.strptime('2019-01', date_format).date()
        req = self.eazy.customers(search_from=req_date)
        self.assertNotIn('No customers could be found', req)

        req_json = json(req)
        for record in req_json['Customers']:
            date_record = record['DateAdded'][0:7]
            date = datetime.strptime(date_record, date_format).date()
            self.assertGreaterEqual(date, req_date)

    def test_search_search_from_year(self):
        date_format = '%Y'
        req_date = datetime.strptime('2019', date_format).date()
        req = self.eazy.customers(search_from=req_date)
        self.assertNotIn('No customers could be found', req)

        req_json = json(req)
        for record in req_json['Customers']:
            date_record = record['DateAdded'][0:4]
            date = datetime.strptime(date_record, date_format).date()
            self.assertGreaterEqual(date, req_date)

    def test_search_date_of_birth_full_date(self):
        # This will fail. This is a limitation of the API.
        date_format = '%Y-%m-%d'
        req_date = datetime.strptime('1990-06-29', date_format).date()
        req = self.eazy.customers(date_of_birth=req_date)
        self.assertNotIn('No customers could be found', req)

        req_json = json(req)
        for record in req_json['Customers']:
            date_record = record['DateOfBirth'][0:10]
            self.assertIsNotNone(date_record)
            date = datetime.strptime(date_record, date_format).date()
            self.assertGreaterEqual(date, req_date)

    def test_search_date_of_birth_from_month(self):
        # This will fail. This is a limitation of the API.
        date_format = '%Y-%m'
        req_date = datetime.strptime('1990-06', date_format).date()
        req = self.eazy.customers(date_of_birth=req_date)
        self.assertNotIn('No customers could be found', req)

        req_json = json(req)
        for record in req_json['Customers']:
            date_record = record['DateOfBirth'][0:7]
            self.assertIsNotNone(date_record)
            date = datetime.strptime(date_record, date_format).date()
            self.assertGreaterEqual(date, req_date)

    def test_search_date_of_birth_from_year(self):
        # This will fail. This is a limitation of the API.
        date_format = '%Y'
        req_date = datetime.strptime('1990', date_format).date()
        req = self.eazy.customers(date_of_birth=req_date)
        self.assertNotIn('No customers could be found', req)

        req_json = json(req)
        for record in req_json['Customers']:
            date_record = record['DateOfBirth'][0:4]
            self.assertIsNotNone(date_record)
            date = datetime.strptime(date_record, date_format).date()
            self.assertGreaterEqual(date, req_date)

    def test_search_customer_ref_full_reference_number(self):
        req = self.eazy.customers(customer_reference='test-000002')
        req_json = json(req)
        for record in req_json['Customers']:
            customer_ref = record['CustomerRef']
            self.assertEqual(customer_ref, 'test-000002')
        self.assertEqual(len(req_json), 1)

    def test_search_customer_ref_partial_ref_number(self):
        # This will fail. This is a limitation of the API.
        req = self.eazy.customers(customer_reference='test')
        try:
            req_json = json(req)
            for record in req_json['Customers']:
                customer_ref = record['CustomerRef']
                self.assertIn('test', customer_ref)
            self.assertGreaterEqual(len(req_json), 1)
        except JSONDecodeError:
            self.fail('json was not returned, no customers were returned.')

    def test_search_customer_first_name_full_name(self):
        req = self.eazy.customers(first_name='test')
        try:
            req_json = json(req)
            for record in req_json['Customers']:
                first_name = record['FirstName']
                self.assertIn('test', first_name.lower())
            self.assertGreaterEqual(len(req_json), 1)
        except JSONDecodeError:
            self.fail('json was not returned, no customers were returned.')

    def test_search_customer_first_name_partial_name(self):
        req = self.eazy.customers(first_name='te')
        try:
            req_json = json(req)
            for record in req_json['Customers']:
                first_name = record['FirstName']
                self.assertIn('te', first_name.lower())
            self.assertGreaterEqual(len(req_json), 1)
        except JSONDecodeError:
            self.fail('json was not returned, no customers were returned.')

    def test_search_customer_surname_full_name(self):
        req = self.eazy.customers(surname='test')
        try:
            req_json = json(req)
            for record in req_json['Customers']:
                first_name = record['Surname']
                self.assertIn('test', first_name.lower())
            self.assertGreaterEqual(len(req_json), 1)
        except JSONDecodeError:
            self.fail('json was not returned, no customers were returned.')

    def test_search_customer_surname_partial_name(self):
        req = self.eazy.customers(surname='te')
        try:
            req_json = json(req)
            for record in req_json['Customers']:
                first_name = record['Surname']
                self.assertIn('te', first_name.lower())
            self.assertGreaterEqual(len(req_json), 1)
        except JSONDecodeError:
            self.fail('json was not returned, no customers were returned.')

    def test_search_customer_company_name_full(self):
        req = self.eazy.customers(company_name='SDK Test Cliemt')
        try:
            req_json = json(req)
            for record in req_json['Customers']:
                first_name = record['CompanyName']
                self.assertEqual('sdk test cliemt', first_name.lower())
            self.assertGreaterEqual(len(req_json), 1)
        except JSONDecodeError:
            self.fail('json was not returned, no customers were returned.')

    def test_search_customer_company_name_partial(self):
        req = self.eazy.customers(company_name='test')
        try:
            req_json = json(req)
            for record in req_json['Customers']:
                first_name = record['CompanyName']
                print(first_name)
                self.assertIn('te', first_name.lower())
            self.assertGreaterEqual(len(req_json), 1)
        except JSONDecodeError:
            self.fail('json was not returned, no customers were returned.')

    def test_search_customer_post_code_full_post_code(self):
        req = self.eazy.customers(post_code='GL52 2NF')
        try:
            req_json = json(req)
            for record in req_json['Customers']:
                post_code = record['AddressDetail']['PostCode']
                self.assertEqual('gl52 2nf', post_code.lower())
            self.assertGreaterEqual(len(req_json), 1)
        except JSONDecodeError:
            self.fail('json was not returned, no customers were returned.')

    def test_search_customer_post_code_partial_post_code(self):
        req = self.eazy.customers(post_code='GL52')
        try:
            req_json = json(req)
            for record in req_json['Customers']:
                post_code = record['AddressDetail']['PostCode']
                self.assertIn('gl52', post_code.lower())
            self.assertGreaterEqual(len(req_json), 1)
        except JSONDecodeError:
            self.fail('json was not returned, no customers were returned.')

    def test_search_customer_account_number_full_number(self):
        req = self.eazy.customers(account_number='45678912')
        try:
            req_json = json(req)
            for record in req_json['Customers']:
                account_number = record['BankDetail']['AccountNumber']
                self.assertEqual('45678912', account_number.lower())
            self.assertEqual(len(req_json), 1)
        except JSONDecodeError:
            self.fail('json was not returned, no customers were returned.')

    def test_search_customer_sort_code_full_number(self):
        req = self.eazy.customers(sort_code='147852')
        try:
            req_json = json(req)
            for record in req_json['Customers']:
                sort_code = record['BankDetail']['BankSortCode']
                self.assertEqual('147852', sort_code.lower())
            self.assertGreaterEqual(len(req_json), 1)
        except JSONDecodeError:
            self.fail('json was not returned, no customers were returned.')

    def test_search_customer_account_holder_name_full_name(self):
        req = self.eazy.customers(account_holder_name='Mr Test Client')
        try:
            req_json = json(req)
            for record in req_json['Customers']:
                account_name = record['BankDetail']['AccountHolderName']
                self.assertEqual('mr test client', account_name.lower())
            self.assertGreaterEqual(len(req_json), 1)
        except JSONDecodeError:
            self.fail('json was not returned, no customers were returned.')

    def test_search_customer_home_phone_full_number(self):
        req = self.eazy.customers(home_phone='01242147852')
        try:
            req_json = json(req)
            for record in req_json['Customers']:
                phone_number = record['HomePhoneNumber']
                self.assertEqual('01242147852', phone_number)
            self.assertEqual(len(req_json), 1)
        except JSONDecodeError:
            self.fail('json was not returned, no customers were returned.')

    def test_search_customer_home_phone_partial_number(self):
        req = self.eazy.customers(home_phone='01242')
        try:
            req_json = json(req)
            for record in req_json['Customers']:
                phone_number = record['HomePhoneNumber']
                self.assertIn('01242', phone_number)
            self.assertGreaterEqual(len(req_json), 1)
        except JSONDecodeError:
            self.fail('json was not returned, no customers were returned.')

    def test_search_customer_mobile_phone_full_number(self):
        req = self.eazy.customers(mobile_phone='07393549789')
        try:
            req_json = json(req)
            for record in req_json['Customers']:
                phone_number = record['MobilePhoneNumber']
                self.assertEqual('07393549789', phone_number)
            self.assertEqual(len(req_json), 1)
        except JSONDecodeError:
            self.fail('json was not returned, no customers were returned.')

    def test_search_customer_mobile_phone_partial_number(self):
        req = self.eazy.customers(mobile_phone='07393')
        try:
            req_json = json(req)
            for record in req_json['Customers']:
                phone_number = record['HomePhoneNumber']
                self.assertIn('01242', phone_number)
            self.assertGreaterEqual(len(req_json), 1)
        except JSONDecodeError:
            self.fail('json was not returned, no customers were returned.')

    def test_search_customer_work_phone_full_number(self):
        req = self.eazy.customers(work_phone='01452365478')
        try:
            req_json = json(req)
            for record in req_json['Customers']:
                phone_number = record['WorkPhoneNumber']
                self.assertEqual('01452365478', phone_number)
            self.assertEqual(len(req_json), 1)
        except JSONDecodeError:
            self.fail('json was not returned, no customers were returned.')

    def test_search_customer_work_phone_partial_number(self):
        req = self.eazy.customers(work_phone='01452')
        try:
            req_json = json(req)
            for record in req_json['Customers']:
                phone_number = record['WorkPhoneNumber']
                self.assertIn('01452', phone_number)
            self.assertGreaterEqual(len(req_json), 1)
        except JSONDecodeError:
            self.fail('json was not returned, no customers were returned.')

    def test_warning_enabled_warning_thrown(self):
        s.warnings['customer_search'] = True
        with self.assertWarns(UserWarning):
            self.eazy.customers()

    def test_warning_disabled_warning_not_thrown(self):
        s.warnings['customer_search'] = False
        try:
            with self.assertWarns(UserWarning):
                self.eazy.customers()
        except AssertionError:
            pass



from ... import main
from json import loads as json
from ...exceptions import InvalidParameterError
from ...exceptions import RecordAlreadyExistsError
from random import getrandbits
import unittest


class Test(unittest.TestCase):
    def setUp(self):
        self.eazy = main.EazySDK().post
        self.r = str(getrandbits(32))

    def test_post_customer_using_valid_data_returns_customer_json_object(self):
        req = self.eazy.customer(
            self.r+'@test.com', 'Mr', self.r, self.r, self.r, '1 Test Road',
            'GL52 2NF', '14785236', '456789', 'Mr Test Test'
        )
        req_json = json(req)
        self.assertIn('CustomerRef', req)
        self.assertIsNotNone(req_json)

    def test_customer_reference_must_be_unique(self):
        with self.assertRaises(RecordAlreadyExistsError) as e:
            self.eazy.customer(
                'test@email.com', 'Mr', 'test-000001', 'Test', 'Test',
                '1 Test Road', 'GL516NB', '14785236', '147852', 'Mr Test Test')

        self.assertIn(
            'A customer with the given customer_reference already exists.'
            ' Please change the customer reference and'
            ' re-submit.', str(e.exception)
        )

    def test_customer_reference_cannot_be_empty(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                'test@email.com', 'Mr', '', 'Test', 'Test',
                '1 Test Road', 'GL516NB', '14785236', '147852', 'Mr Test Test')

        self.assertIn(
            'cannot be empty.', str(e.exception)
        )

    def test_post_code_too_short_throws_error(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                'test@email.com', 'Mr', 'test-000001', 'Test', 'Test',
                '1 Test Road', 'G', '14785236', '147852', 'Mr Test Test')

        self.assertIn(
            'G is not formatted as a UK post code. Please check the post code'
            ' and re-submit.', str(e.exception)
        )

    def test_post_code_too_long_throws_error(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                'test@email.com', 'Mr', 'test-000001', 'Test', 'Test',
                '1 Test Road', 'GL516NBB', '14785236', '147852',
                'Mr Test Test')

        self.assertIn(
            'GL516NBB is not formatted as a UK post code. Please check the'
            ' post code and re-submit.', str(e.exception)
        )

    def test_post_code_must_contain_letters(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                'test@email.com', 'Mr', 'test-000001', 'Test', 'Test',
                '1 Test Road', '99501', '14785236', '147852',
                'Mr Test Test')

        self.assertIn(
            '99501 is not formatted as a UK post code. Please check the'
            ' post code and re-submit.', str(e.exception)
        )

    def test_post_code_must_contain_numbers(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                'test@email.com', 'Mr', 'test-000001', 'Test', 'Test',
                '1 Test Road', 'GLQQIAN', '14785236', '147852',
                'Mr Test Test')

        self.assertIn(
            'GLQQIAN is not formatted as a UK post code. Please check the'
            ' post code and re-submit.', str(e.exception)
        )

    def test_post_code_cannot_be_empty(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                'test@email.com', 'Mr', 'test-000001', 'Test', 'Test',
                '1 Test Road', '', '14785236', '147852',
                'Mr Test Test')

        self.assertIn(
            'cannot be empty.', str(e.exception)
        )

    def test_email_must_contain_at_symbol(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                'testemail.com', 'Mr', 'test-000001', 'Test', 'Test',
                '1 Test Road', 'GL51 6NB', '14785236', '147852',
                'Mr Test Test')

        self.assertIn(
            'is not a valid email address. Please check the email address'
            ' and re-submit.', str(e.exception)
        )

    def test_email_must_contain_tld(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                'test@emailcom', 'Mr', 'test-000001', 'Test', 'Test',
                '1 Test Road', 'GL51 6NB', '14785236', '147852',
                'Mr Test Test')

        self.assertIn(
            'is not a valid email address. Please check the email address'
            ' and re-submit.', str(e.exception)
        )

    def test_email_cannot_contain_special_characters_in_domain(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                'test@e!m&a$il.com', 'Mr', 'test-000001', 'Test', 'Test',
                '1 Test Road', 'GL51 6NB', '14785236', '147852',
                'Mr Test Test')

        self.assertIn(
            'is not a valid email address. Please check the email address'
            ' and re-submit.', str(e.exception)
        )

    def test_email_cannot_be_empty(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                '', 'Mr', 'test-000001', 'Test', 'Test',
                '1 Test Road', 'GL51 6NB', '14785236', '147852',
                'Mr Test Test')

        self.assertIn(
            'cannot be empty.', str(e.exception)
        )

    def test_account_number_must_not_contain_letters(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                'test@email.com', 'Mr', 'test-000001', 'Test', 'Test',
                '1 Test Road', 'GL51 6NB', '1478523a', '147852',
                'Mr Test Test')

        self.assertIn(
            'is not formatted as a UK bank account number. UK bank'
            ' account numbers are 8 digits long. Please check the bank'
            ' account number and re-submit.', str(e.exception)
        )

    def test_account_number_must_not_be_too_short(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                'test@email.com', 'Mr', 'test-000001', 'Test', 'Test',
                '1 Test Road', 'GL51 6NB', '14785', '147852',
                'Mr Test Test')

        self.assertIn(
            'is not formatted as a UK bank account number. UK bank'
            ' account numbers are 8 digits long. Please check the bank'
            ' account number and re-submit.', str(e.exception)
        )

    def test_account_number_must_not_be_too_long(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                'test@email.com', 'Mr', 'test-000001', 'Test', 'Test',
                '1 Test Road', 'GL51 6NB', '1478514717', '147852',
                'Mr Test Test')

        self.assertIn(
            'is not formatted as a UK bank account number. UK bank'
            ' account numbers are 8 digits long. Please check the bank'
            ' account number and re-submit.', str(e.exception)
        )

    def test_account_number_must_not_be_empty(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                'test@email.com', 'Mr', 'test-000001', 'Test', 'Test',
                '1 Test Road', 'GL51 6NB', '', '147852',
                'Mr Test Test')

        self.assertIn(
            'cannot be empty', str(e.exception)
        )

    def test_sort_code_must_not_contain_letters(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                'test@email.com', 'Mr', 'test-000001', 'Test', 'Test',
                '1 Test Road', 'GL51 6NB', '14785231', '14785a',
                'Mr Test Test')

        self.assertIn(
            'is not formatted as a UK sort code. UK sort codes are 6'
            ' digits long. Make sure to not include any hyphens. Please check'
            ' the sort code and re-submit.', str(e.exception)
        )

    def test_sort_code_must_not_be_too_short(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                'test@email.com', 'Mr', 'test-000001', 'Test', 'Test',
                '1 Test Road', 'GL51 6NB', '14785231', '14785',
                'Mr Test Test')

        self.assertIn(
            'is not formatted as a UK sort code. UK sort codes are 6'
            ' digits long. Make sure to not include any hyphens. Please check'
            ' the sort code and re-submit.', str(e.exception)
        )

    def test_sort_code_must_not_be_too_long(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                'test@email.com', 'Mr', 'test-000001', 'Test', 'Test',
                '1 Test Road', 'GL51 6NB', '14785231', '1478521',
                'Mr Test Test')

        self.assertIn(
            'is not formatted as a UK sort code. UK sort codes are 6'
            ' digits long. Make sure to not include any hyphens. Please check'
            ' the sort code and re-submit.', str(e.exception)
        )

    def test_sort_code_must_not_contain_hyphens(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                'test@email.com', 'Mr', 'test-000001', 'Test', 'Test',
                '1 Test Road', 'GL51 6NB', '14785231', '14-78-52',
                'Mr Test Test')

        self.assertIn(
            'is not formatted as a UK sort code. UK sort codes are 6'
            ' digits long. Make sure to not include any hyphens. Please check'
            ' the sort code and re-submit.', str(e.exception)
        )

    def test_sort_code_cannot_be_empty(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                'test@email.com', 'Mr', 'test-000001', 'Test', 'Test',
                '1 Test Road', 'GL51 6NB', '14785231', '',
                'Mr Test Test')

        self.assertIn(
            'cannot be empty', str(e.exception)
        )

    def test_account_holder_name_cannot_be_too_long(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                'test@email.com', 'Mr', 'test-000001', 'Test', 'Test',
                '1 Test Road', 'GL51 6NB', '14785231', '147852',
                'Mr Test Test Test Test Test')

        self.assertIn(
            'is not formatted as an account holder name. Account holder names'
            ' must be between 3 and 18 characters, contain only capital'
            ' letters (A-7), ampersands (&), hyphens (-), forward slashes'
            ' (/), and spaces ( ).', str(e.exception)
        )

    def test_account_holder_name_cannot_be_too_short(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                'test@email.com', 'Mr', 'test-000001', 'Test', 'Test',
                '1 Test Road', 'GL51 6NB', '14785231', '147852',
                'Mr')

        self.assertIn(
            'is not formatted as an account holder name. Account holder names'
            ' must be between 3 and 18 characters, contain only capital'
            ' letters (A-7), ampersands (&), hyphens (-), forward slashes'
            ' (/), and spaces ( ).', str(e.exception)
        )

    def test_account_holder_name_cannot_contain_special_characters(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                'test@email.com', 'Mr', 'test-000001', 'Test', 'Test',
                '1 Test Road', 'GL51 6NB', '14785231', '147852',
                'Mr !^&()$"!')

        self.assertIn(
            'is not formatted as an account holder name. Account holder names'
            ' must be between 3 and 18 characters, contain only capital'
            ' letters (A-7), ampersands (&), hyphens (-), forward slashes'
            ' (/), and spaces ( ).', str(e.exception)
        )

    def test_account_holder_name_cannot_be_empty(self):
        with self.assertRaises(InvalidParameterError) as e:
            self.eazy.customer(
                'test@email.com', 'Mr', 'test-000001', 'Test', 'Test',
                '1 Test Road', 'GL51 6NB', '14785231', '147852',
                '')

        self.assertIn(
            'cannot be empty', str(e.exception)
        )

    def test_parameter_not_allowed_error(self):
        with self.assertRaises(TypeError):
            self.eazy.customer(not_an_arg='test@email.com')




from pathlib import Path
from datetime import datetime
from ..settings import Settings as s
from datetime import timedelta
from requests import get


base_path = Path(__file__).parent
bank_holidays_file = (base_path / '../includes/holidays.csv').resolve()


def check_working_days_in_future(number_of_days):
    """ Take X number of working days, and calculate the number of
    working days X is in the future from today. Call
    read_bank_holiday_file_and_check_if_update_needed() to get a list
    of bank holidays.

    :Args:
    number_of_days - The number of days in the future X is
    """
    holidays = read_bank_holiday_file_and_check_if_update_needed()
    working_days = 0
    calendar_days = 0
    today_datetime = datetime.now()
    today_date = today_datetime.date()
    # We want to keep today_date constant for raising errors
    start_date = today_date

    # Check if today is a weekday, if it is not, go to next weekday
    if today_date.isoweekday() == 6 or today_date.isoweekday() == 7 or \
            str(today_date) in holidays:
        # Should never need 10 iterations. We want the next working day
        for i in range(10):
            iter_date = today_date + timedelta(i)
            if iter_date.isoweekday() in range(1, 5) \
                    and str(iter_date) not in holidays:
                start_date += timedelta(i)
                break

    while working_days <= (number_of_days - 1):
        working_date = start_date + timedelta(calendar_days)
        if working_date.isoweekday() != 6 and working_date.isoweekday() != 7 \
                and str(working_date) not in holidays:
            working_days += 1
        calendar_days += 1
    final_date = start_date + timedelta(calendar_days)

    while final_date.isoweekday() == 6 or final_date.isoweekday() == 7 \
            or str(final_date) in holidays:
        final_date += timedelta(1)
        calendar_days += 1
    return final_date


def read_bank_holiday_file_and_check_if_update_needed():
    """ Read the bank holidays file and return a list of all bank
    bank holidays for the current year. If the file has not been updated
    in the pre_determined number of days in the settings file, read
    the bank_holidays.json file and get a list of all bank holidays in
    the current year or later.
    """
    # We need the datetime to access the year element
    today_datetime = datetime.now()
    today_date = today_datetime.date()
    holidays = []
    with open(bank_holidays_file, 'a+') as f:
        f.seek(0)
        try:
            # Check the first line of the file to check last update
            text_date = f.readline().strip('\n')

            # The file stores the date as a string. We need a Date object
            date_formatting = '%Y-%m-%d'
            last_update_date = datetime.strptime(
                text_date, date_formatting
            ).date()
            date_difference = today_date - last_update_date
            day_difference = date_difference.days

            if day_difference >= s.other['bank_holidays_update_days']:
                print(
                    'The bank holidays file has not been updated in over %s'
                    ' days. Updating......' % day_difference
                )
                new_holidays = get('https://www.gov.uk/bank-holidays.json')
                holidays_json = new_holidays.json()['england-and-wales'] \
                    ['events']

                for date in holidays_json:
                    # Add  bank holidays from or after the current year
                    if int(date['date'][0:4]) >= today_datetime.year:
                        holidays.append(date['date'])
                    else:
                        continue
                    update_bank_holidays_file(holidays)
            else:
                for line in f:
                    holidays.append(line.strip('\n'))

            return holidays
        except:
            new_holidays = get('https://www.gov.uk/bank-holidays.json')
            holidays_json = new_holidays.json()['england-and-wales']['events']

            for date in holidays_json:
                # Add  bank holidays from or after the current year
                if int(date['date'][0:4]) >= today_datetime.year:
                    holidays.append(date['date'])
                else:
                    continue
                update_bank_holidays_file(holidays)
        return holidays


def update_bank_holidays_file(bank_holiday_list):
    """ Write the bank_holiday_list to the bank_holidays.csv file,
    prepending it with todays date, which will be used for updating the
    file in the future.

    :Args:
    number_of_days - The number of days in the future X is
    """
    with open(bank_holidays_file, 'w') as f:
        # Update the header
        f.write(str(datetime.now().date()))
        for date in bank_holiday_list:
            f.write('\n%s' % date)
    return 'Updated bank holidays file.'



from ... import main
from ...exceptions import ResourceNotFoundError
import unittest


class Test(unittest.TestCase):
    def setUp(self):
        self.eazy = main.EazySDK().post

    def test_restart_contract(self):
        req = self.eazy.restart_contract(
            'a899ced6-a601-4146-92f7-5c8ee40bbf93', 'Until further notice',
            'Switch to further notice', payment_day_in_month=15,
            payment_month_in_year=6
        )
        self.assertIn('Contract restarted', req)

    def test_restart_contract_contract_not_expired_returns_error(self):
        req = self.eazy.restart_contract(
            'a899ced6-a601-4146-92f7-5c8ee40bbf93', 'Until further notice',
            'Switch to further notice', payment_day_in_month=15,
            payment_month_in_year=6
        )
        self.assertIn('is not expired, no action has been taken', req)

    def test_archive_contract_incorrect_contract_returns_error(self):
        with self.assertRaises(ResourceNotFoundError) as e:
            self.eazy.restart_contract(
                '99ced6-a601-4146-92f7-5c8ee40bbf93', 'Until further notice',
                'Switch to further notice', payment_day_in_month=15,
                payment_month_in_year=6
            )
        self.assertIn('resource could not be found', str(e.exception))